# 公式

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/robjects_formula.html](https://rpy2.readthedocs.io/en/version_2.8.x/robjects_formula.html)

对于建模和绘图等任务，R 公式可以是一种简洁但可读的表达所需内容的方式。

在 R 中，通常看起来：

```
x <- 1:10
y <- x + rnorm(10, sd=0.2)

fit <- lm(y ~ x)

```

在对`lm`的调用中，参数是一个`formula`，使用 x 可以像 _ 模型 y 一样读取。公式是**r**语言对象，公式中的术语在其定义的环境中进行计算。如果没有进一步的规范，该环境就是创建公式的环境。_

类`robjects.Formula`表示**r**公式。

```
import array
from rpy2.robjects import IntVector, Formula
from rpy2.robjects.packages import importr
stats = importr('stats')

x = IntVector(range(1, 11))
y = x.ro + stats.rnorm(10, sd=0.2)

fmla = Formula('y ~ x')
env = fmla.environment
env['x'] = x
env['y'] = y

fit = stats.lm(fmla)

```

该方法的一个缺点是，漂亮地打印`fit`对象与在**r**中工作时所期望的效果相当：现在`call`项显示用于执行 fit 的函数的代码。

如果仍想避免污染 R 全局环境，答案是在定义函数的环境中评估 R 调用。

```
from rpy2.robjects import Environment

eval_env = Environment()
eval_env['fmla'] = fmla
base = importr('base')

fit = base.eval.rcall(base.parse(text = 'lm(fmla)'), stats._env)

```

其他选项包括：

*   实时评估 R 代码，使模型拟合函数在 R 中有一个符号。

    ```
    fit = robjects.r('lm(%s)' %fmla.r_repr())

    ```

*   在定义了所有符号的地方计算 R 代码