> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/vector.html](https://rpy2.readthedocs.io/en/version_2.8.x/vector.html)

# 向量和数组

除了函数和环境之外，R 用户与之交互的大多数对象都是类似向量的。例如，这意味着任何标量实际上都是长度为 1 的向量。

类`Vector`有一个构造函数：

```
>>> x = robjects.Vector(3)

```

## 创建向量

创建向量可以通过 r 或 python 实现。

当向量是从 r 创建的时候，我们不应该担心它们会被暴露出来，因为它们应该被[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")暴露出来。

当您想要从 python 创建一个向量时，可以使用类`Vector`或方便类`IntVector`、`FloatVector`、`BoolVector`、`StrVector`。

日期或时间点序列可以存储在`POSIXlt`或`POSIXct`对象中。两者都可以从[`time.struct_time`](http://docs.python.org/library/time.html#time.struct_time "(in Python v2.7)")对象的 python 序列或从 r 对象创建。

2.2.0 版新增：日期或时间点矢量

### 因子服务器

R 的因子有点特别：它们的目标是表示一个内存高效的标签向量，为了实现这一点，它们被实现为与标签向量（可能更短）相关联的整数向量。每个整数表示标签在相关标签向量中的位置。

例如，以下标签向量

<colgroup><col width="17%"> <col width="17%"> <col width="17%"> <col width="17%"> <col width="17%"> <col width="17%"></colgroup> 
| 一 | B | a | b | b | C |

将成为

<colgroup><col width="17%"> <col width="17%"> <col width="17%"> <col width="17%"> <col width="17%"> <col width="17%"></colgroup> 
| 1 个 | 二 | 1 | 2 | 2 | 三 |

和

<colgroup><col width="33%"> <col width="33%"> <col width="33%"></colgroup> 
| a | b | c |

```
>>> sv = ro.StrVector('ababbc')
>>> fac = ro.FactorVector(sv)
>>> print(fac)
[1] a b a b b c
Levels: a b c
>>> tuple(fac)
(1, 2, 1, 2, 2, 3)
>>> tuple(fac.levels)
('a', 'b', 'c')

```

因为`FactorVector`是一个带有附加元数据（级别）的`IntVector`，所以从`IntVector`获取项目时，获取项目的 python 样式没有改变。这样做的结果是，有关这些级别的信息就会丢失。

```
>>> item_i = 0
>>> fac[item_i]
1

```

获取与项目对应的级别需要使用`levels`：

```
>>> fac.levels[fac[item_i] - 1]
'a'

```

警告

不要忘记将`FactorVector`中的值减去一。python 中的索引从零开始，而 indexing r 从 1 开始，恢复项目的级别需要在两者之间进行调整。

当从`FactorVector`提取元素时，一个合理的默认值可能是使用 R 样式的提取（请参见[提取项](#robjects-extracting)），因为它保留整数/字符串对偶性。

## 正在提取项目

由于 python 和 r 在多个点上存在差异（索引号从零开始/从一开始，负索引号表示从末尾的/_ 索引除 _ 之外的所有内容），因此提取序列/向量的元素可能成为一个棘手的问题，名称不能/可以用于 ttting）。

为了解决这个问题，通过两个不同的路径提供了 python 方法和 r 方法。

### 提取，python 样式

python`__getitem__()`方法的行为类似于 python 用户对向量的期望（索引从零开始）。

```
>>> x = robjects.r.seq(1, 5)
>>> tuple(x)
(1, 2, 3, 4, 5)
>>> x.names = robjects.StrVector('abcde')
>>> print(x)
a b c d e
1 2 3 4 5
>>> x[0]
1
>>> x[4]
5
>>> x[-1]
5

```

### 提取，R 样式

通过分别代表 R 函数 _[_ 和 _[[_ 的两个删除程序 _RX_ 和 _RX2_ 授予对 R 样式提取/子集的访问权。

总之，R 型提取具有以下特点：

*   索引从一开始
*   subset on 的参数可以是
    *   整数（负整数表示元素的排除）
    *   布尔运算
    *   字符串（每当矢量的元素有 _ 名称 _ 时）

```
>>> print(x.rx(1))
[1] 1
>>> print(x.rx('a'))
a
1

```

R 可以一次提取几个元素：

```
>>> i = robjects.IntVector((1, 3))
>>> print(x.rx(i))
[1] 1 3
>>> b = robjects.BoolVector((False, True, False, True, True))
>>> print(x.rx(b))
[1] 2 4 5

```

当布尔提取向量的长度小于该向量时，将根据需要展开（在 R 中称为`recycling rule`）：

```
>>> print(x.rx(True))
1:5
>>> b = robjects.BoolVector((False, True))
>>> print(x.rx(b))
[1] 2 4

```

在 R 中，负指数被理解为元素排除。

```
>>> print(x.rx(-1))
2:5
>>> i = robjects.IntVector((-1, -3))
>>> print(x.rx(i))
[1] 2 4 5

```

最后一个例子也可以写成：

```
>>> i = - robjects.IntVector((1, 3)).ro
>>> print(x.rx(i))
[1] 2 4 5

```

这个提取系统非常有表现力，因为它允许对数据分析中非常常见的任务（如重新排序和随机抽样）进行非常简单的写入。

```
>>> from rpy2.robjects.packages import importr
>>> base = importr('base')
>>> x = robjects.IntVector((5,3,2,1,4))
>>> o_i = base.order(x)
>>> print(x.rx(o_i))
[1] 1 2 3 4 5
>>> rnd_i = base.sample(x)
>>> x_resampled = x.rx(o_i)

```

r 运算符是向量运算，该运算符应用于向量中的每个元素。这可用于构建提取索引。

```
>>> i = x.ro > 3 # extract values > 3
>>> i = (x.ro >= 2 ).ro & (x.ro <= 4) # extract values between 2 and 4

```

（有关 r 运算符的更多信息，请参阅[operators](#robjects-operationsdelegator)一节）。

R/S 也有一些特殊性，其中有些人看到一致性问题。例如，尽管索引从 1 开始，但 0 上的索引不会返回 _ 索引越界 _ 错误，而是返回长度为 0 的向量：

```
>>> print(x.rx(0))
integer(0)

```

## 分配项目

### 指定，python 样式

由于向量是以 python 可变序列的形式公开的，因此该分配与常规的 python 列表一样有效。

```
>>> x = robjects.IntVector((1,2,3))
>>> print(x)
[1] 1 2 3
>>> x[0] = 9
>>> print(x)
[1] 9 2 3

```

在 R 向量中，可以将 _ 命名为 _，即向量的元素有一个名称。尤其是 R 列表的情况。使用方法`Vector.index()`可以很容易地根据名称进行分配，如下所示。

```
>>> x = robjects.ListVector({'a': 1, 'b': 2, 'c': 3})
>>> x[x.names.index('b')] = 9

```

注意

`Vector.index()`在向量长度上具有线性复杂性；如果满足性能问题，则应记住这一点。

### 分配，R 样式

两种语言之间的差异需要很少的适应，并且在外观上稍微复杂一些。如果其他基于 python 的系统用于表示（主要是数字）数据结构，则首选`numpy`，我们将鼓励其中一个系统通过这些结构公开我们的 rpy2 r 对象。

以前使用的属性 _rx_ 和 _rx2_ 可以再次使用：

```
>>> x = robjects.IntVector(range(1, 4))
>>> print(x)
[1] 1 2 3
>>> x.rx[1] = 9
>>> print(x)
[1] 9 2 3

```

为了与 R 完全兼容，参数可以命名（并作为[`dict`](http://docs.python.org/library/stdtypes.html#dict "(in Python v2.7)")或`rpy2.rlike.container.TaggedList`传递）。

```
>>> x = robjects.ListVector({'a': 1, 'b': 2, 'c': 3})
>>> x.rx2[{'i': x.names.index('b')}] = 9

```

## 缺少值

任何有实际数据分析经验的人都知道有些数据可能会丢失。在 S/SPlus/R 特殊的 _ 中，可以在数据向量中使用 na_ 值来表示该事实，并且[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")为那些可用的数据对象创建别名`NA_Logical`、`NA_Real`、`NA_Integer`、`NA_Character`、`NA_Complex`。

```
>>> x = robjects.IntVector(range(3))
>>> x[0] = robjects.NA_Integer
>>> print(x)
[1] NA  1  2

```

NA 类型的转换是在项目级别完成的，返回指向相应 NA singleton 类的指针。

```
>>> x[0] is robjects.NA_Integer
True
>>> x[0] == robjects.NA_Integer
True
>>> [y for y in x if y is not robjects.NA_Integer]
[1, 2]

```

Note

`NA_Logical`是 r 的 _na_ 的别名。

Note

NA 对象是从相应的[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")对象导入的。

## 操作员

对两个向量的数学运算：以下运算在 r 中按元素进行，必要时尽可能多地回收最短的向量。

为了向 Python 公开这一点，为类似向量的对象提供了委派属性`ro`。

<colgroup><col width="37%"> <col width="63%"></colgroup> 
| 蟒蛇 | R |
| --- | --- |
| `+` | `+` |
| `-` | `-` |
| `*` | `*` |
| `/` | `/` |
| `**` | `**`或`^` |
| `or` | `&#124;` |
| `and` | `&` |
| `&lt;` | `&lt;` |
| `&lt;=` | `&lt;=` |
| `==` | `==` |
| `!=` | `!=` |

```
>>> x = robjects.r.seq(1, 10)
>>> print(x.ro + 1)
2:11

```

Note

在 python 中，对两个序列使用操作符`+`将它们连接起来，并且此行为已得到保护：

```
>>> print(x + 1)
[1]  1  2  3  4  5  6  7  8  9 10  1

```

Note

不能在 python（至少 2.5 版之前）中重新定义布尔运算符`not`，并且不能使其行为模仿 r 的行为。

## 姓名

`R`矢量可以为所有或部分元素命名。属性`names`可用于获取或设置这些名称。

```
>>> x = robjects.r.seq(1, 5)
>>> x.names = robjects.StrVector('abcde')
>>> x.names[0]
'a'
>>> x.names[0] = 'z'
>>> tuple(x.names)
('z', 'b', 'c', 'd', 'e')

```

## `Array`

在`R`中，数组只是具有维度属性的向量。这一事实反映在类层次结构中，其中`robjects.Array`继承自`robjects.Vector`。

## `Matrix`

`Matrix`是`Array`的特例。与数组一样，必须记住这只是一个具有维度属性（行数、列数）的向量。

```
>>> m = robjects.r.matrix(robjects.IntVector(range(10)), nrow=5)
>>> print(m)
     [,1] [,2]
[1,]    0    5
[2,]    1    6
[3,]    2    7
[4,]    3    8
[5,]    4    9

```

Note

在 _r_ 中，矩阵是按列主顺序排列的，尽管构造函数`matrix()`接受一个布尔参数 _byrow_，如果为 true，将像按行主顺序排列一样构建矩阵 _。_

### 矩阵计算

正则操作符在底层向量上对元素进行操作。

```
>>> m = robjects.r.matrix(robjects.IntVector(range(4)), nrow=2)
>>> print(m.ro + 1)
     [,1] [,2]
[1,]    1    3
[2,]    2    4

```

有关运算符的详细信息，请参阅[运算符](#robjects-operationsdelegator)。

矩阵乘法可用作`Matrix.dot()`，换位为`Matrix.transpose()`。通过显式名称的方法，还可以进行交叉积、特征值计算和奇异值分解等常见操作。

```
>>> print( m.crossprod(m) )
     [,1] [,2]
[1,]    1    3
[2,]    3   13
>>> print( m.transpose().dot(m) )
     [,1] [,2]
[1,]    1    3
[2,]    3   13

```

### 提取

提取仍然可以执行 python 样式或 r 样式。

```
>>> m = ro.r.matrix(ro.IntVector(range(2, 8)), nrow=3)
>>> print(m)
     [,1] [,2]
[1,]    2    5
[2,]    3    6
[3,]    4    7
>>> m[0]
2
>>> m[5]
7
>>> print(m.rx(1))
[1] 2
>>> print(m.rx(6))
[1] 7

```

矩阵是二维数组，可以根据两个索引提取元素：

```
>>> print(m.rx(1, 1))
[1] 2
>>> print(m.rx(3, 2))
[1] 7

```

提取整行或整列可以通过用`True`或`False`替换索引号来实现。

提取第一列：

```
>>> print(m.rx(True, 1))

```

提取第二行：

```
>>> print(m.rx(2, True))

```

## `DataFrame`

数据帧是 R 中表示要分析的数据的常用方法。

数据帧可以看作是数据的表格表示，每列一个变量，每行一个数据点。每列都是一个 r 向量，这意味着一个给定列中的所有元素都有一个类型，并且允许不同列中可能有不同的类型。

例如，如果我们考虑茶碱在不同受试者体内的药代动力学的 TRE 数据，数据表可能如下所示：

<colgroup><col width="28%"> <col width="24%"> <col width="16%"> <col width="16%"> <col width="16%"></colgroup> 
| 主题 | 重量 | 剂量 | 时间 | 混凝土 |
| --- | --- | --- | --- | --- |
| 1 | 79.6 条 | 4.02 节 | 0.00 美元 | 0.74 分 |
| 1 | 79.6 | 4.02 | 0.25 分 | 2.84 条 |
| 1 | 79.6 | 4.02 | 0.57 分 | 6.57 条 |
| 2 | 第 72.4 条 | 4.40 条 | 7.03 日 | 5.40 条 |
| …… | ... | ... | ... | ... |

这样的数据表示与关系数据库中的表有相同之处：变量或列之间的结构由其他列给出。在上面的示例中，按主题分组的度量值由列 _subject_ 给出。

在[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")中，`DataFrame`表示`R`类`data.frame`。

### 创建对象

创建`DataFrame`可以通过以下方式完成：

*   使用类的构造函数
*   通过 r 创建 data.frame
*   使用实例方法`from_csvfile()`从文件中读取数据

`DataFrame`构造函数接受`rinterface.SexpVector`（其中`typeof`等于 _vecsxp_，即 R`list`）或任何实现方法`items()`的 python 对象（例如[`dict`](http://docs.python.org/library/stdtypes.html#dict "(in Python v2.7)")或`rpy2.rlike.container.OrdDict`）。

空`data.frame`：

```
>>> dataf = robjects.DataFrame({})

```

`data.frame`有两列（但结果`DataFrame`中列的顺序不能与声明它们的顺序不同）：

```
>>> d = {'a': robjects.IntVector((1,2,3)), 'b': robjects.IntVector((4,5,6))}
>>> dataf = robject.DataFrame(d)

```

要创建`DataFrame`并确定 clumn 顺序，可以使用有序字典：

```
>>> import rpy2.rlike.container as rlc
>>> od = rlc.OrdDict([('value', robjects.IntVector((1,2,3))),
                      ('letter', robjects.StrVector(('x', 'y', 'z')))])
>>> dataf = robjects.DataFrame(od)
>>> print(dataf.colnames)
[1] "letter" "value"

```

在 r 中创建 data.frame 可以通过多种方式实现，因为许多 r 函数都会返回一个`data.frame`，比如函数`data.frame()`。

Note

创建`DataFrame`时，字符串向量由 r 自动转换为类`Factor`的实例。通过将调用包装到 R 基函数 i 中可以防止此行为。

```
from rpy2.robjects.vectors import DataFrame, StrVector
from rpy2.robjects.packages import importr
base = importr('base')
dataf = DataFrame({'string': base.I(StrVector('abbab')),
                   'factor': StrVector('abbab')})

```

这里的`DataFrame``dataf`现在有两列，一列作为`Factor`，另一列作为`StrVector`

```
>>> dataf.rx2('string')
<StrVector - Python:0x95fe5ec / R:0x9646ea0>
>>> dataf.rx2('factor')
<FactorVector - Python:0x95fe86c / R:0x9028138>

```

### 提取元素

这里再次说明，Python 的`__getitem__()`将作为一个 Python 程序员工作，希望它：

```
>>> len(dataf)
2
>>> dataf[0]
<Vector - Python:0x8a58c2c / R:0x8e7dd08>

```

`DataFrame`由列组成，每列可能具有不同的类型：

```
>>> [column.rclass[0] for column in dataf]
['factor', 'integer']

```

使用 r 样式的元素访问有点丰富，而 _rx2_ 访问器比以前更重要。

与上面的 python`__getitem__()`一样，在一个索引上提取选择列：

```
>>> dataf.rx(1)
<DataFrame - Python:0x8a584ac / R:0x95a6fb8>
>>> print(dataf.rx(1))
  letter
1      x
2      y
3      z

```

请注意，结果本身就是类`DataFrame`。要将列作为向量，请使用 _rx2_：

```
>>> dataf.rx2(1)
<Vector - Python:0x8a4bfcc / R:0x8e7dd08>
>>> print(dataf.rx2(1))
[1] x y z
Levels: x y z

```

由于数据帧是类似表格的结构，因此可以将其视为二维数组，因此可以在两个索引上提取。

```
>>> subdataf = dataf.rx(1, True)
>>> print(subdataf)
  letter value
1      x     1
>>> rows_i <- robjects.IntVector((1,3))
>>> subdataf = dataf.rx(rows_i, True)
>>> print(subdataf)
  letter value
1      x     1
3      z     3

```

最后一个例子在 R 中非常常见。这里，索引向量 _ 行 _I_ 用于获取`DataFrame`的一个子集。

### python 文档字符串