> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/robjects_convert.html](https://rpy2.readthedocs.io/en/version_2.8.x/robjects_convert.html)

# 将 rpy2 对象映射到任意 python 对象

注意

在转换模式和非转换模式之间切换，使用 rpy-1.x 时经常出现的操作不再需要使用`rpy2`。

在`rpy2`中遵循的方法有两个级别（`rinterface`和`robjects`），转换函数有助于在它们之间移动。

## 协议

在较低的级别（[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")），暴露 R 对象的 rpy2 对象实现了 Python 协议，使它们对 Python 程序员来说尽可能自然。使用它们，可以将它们作为参数传递给许多非 rpy2 函数，而不需要转换。

r 向量被映射到在序列协议中实现方法`__getitem__()`/`__setitem__()`的 python 对象，因此元素可以很容易地被访问。它们还实现了 python 缓冲区协议，允许在`numpy`函数中使用它们，而不需要数据复制或转换。

r 函数被映射到实现`__call__()`的 python 对象，因此可以像调用函数一样调用它们。

r 环境被映射到在映射协议中实现`__getitem__()`/`__setitem__()`的 python 对象，因此可以类似地在 python[`dict`](http://docs.python.org/library/stdtypes.html#dict "(in Python v2.7)")中访问元素。

Note

`rinterface`级别主要在 C 中实现，用于桥接 Python 和 R C-API。定制它并非易事。

## 转换

在它的高级接口中`rpy2`使用的是一个转换系统，它的任务是在以下 3 种表示形式之间转换对象：-较低级别的接口到 r（[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")级别），-较高级别的接口到 r（[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")级别）-其他（没有`rpy2`）repr 陈述

> 例如，如果希望将所有 python`tuple`转换为 r`character`向量（字符串的一维数组），正如`rpy2`的低级接口所显示的那样，该函数将如下所示：

```
from rpy2.rinterface import StrSexpVector
def tuple_str(tpl):
    res = StrSexpVector(tpl)
    return res

```

### 转换器对象

类`rpy2.robjects.conversion.Converter`将此类转换函数分组为一个对象。

我们上面定义的转换函数可以注册如下：

```
from rpy2.robjects.conversion import Converter
my_converter = Converter('my converter')
my_converter.py2ri.register(tuple, tuple_str)

```

转换器对象是相加的，这是创建简单转换规则组合的简单方法。例如，创建一个将上述规则添加到默认转换规则的转换器：

```
from rpy2.robjects import default_converter
default_converter + my_converter

```

### 本地转换规则

转换规则可以全局自定义（参见`Customizing the conversion`一节），也可以通过使用本地转换器作为上下文管理器进行自定义。当试验或希望转换系统的特定行为受到时间限制时，建议使用后者。

如果要更改`rpy2`当前拒绝处理未指定类型的序列，可以使用此示例。

以下代码引发了一个`rpy2`不知道如何处理 python 序列的错误。

```
x = (1,2,'c')

from rpy2.robjects.packages import importr
base = importr('base')

# error here:
res = base.paste(x, collapse="-")

```

这可以通过使用转换器作为默认转换方案的补充来更改：

```
from rpy2.robjects import default_converter
from rpy2.robjects.conversion import Converter, localconverter
with localconverter(default_converter + my_converter) as cv:
    res = base.paste(x, collapse="-")

```

### `ri2ro()`

在这个级别，转换是在较低级别（[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")）对象和较高级别（[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")）对象之间进行的。此方法是在`functools.singledispatch()`中实现的通用方法（使用 python 2，`singledispatch.singledispatch()`）。

### `ri2py()`

在这个级别上，转换是在较低级别（[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")）对象和任何对象之间进行的（假设非 rpy2 是可以进行的转换）。此方法是在`functools.singledispatch()`（使用 python 2，`singledispatch.singledispatch()`）中实现的通用方法。

例如，`numpy`对象的可选转换方案将尽可能返回 numpy 数组。

Note

`robjects`-级别对象也隐式地是`rinterface`-级别对象，因为它们的类定义中存在继承关系，但反过来则不是真的。`robjects`级别是一个更高级别的抽象，旨在简化从 python 使用 r 的过程（尽管可能会牺牲性能）。

### `p2ri()`

在这个级别上，转换是在（大概）非 rpy2 对象和 rpy2 较低级别（[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")）之间进行的。

此方法是在`functools.singledispatch()`（使用 python 2，`singledispatch.singledispatch()`）中实现的通用方法。

### 自定义转换

作为一个例子，假设当 r 数值向量的长度为 1 时，我们希望返回原子值。这只是编写一个新的函数`ri2py`来处理这个问题，如下所示：

```
import rpy2.robjects as robjects
from rpy2.rinterface import SexpVector

@robjects.conversion.ri2ro.register(SexpVector)
def my_ri2ro(obj):
    if len(obj) == 1:
        obj = obj[0]
    return obj

```

然后我们可以用以下方法进行测试：

```
>>> pi = robjects.r.pi
>>> type(pi)
<type 'float'>

```

在编写`singledispath()`时，没有提供到`unregister`的方法。在不重新启动 python 的情况下删除附加的转换规则留给读者作为练习。

警告

示例稍微弯曲了 rpy2 规则，因为它使用的是`ri2ro`，而当 r 向量长度为 1 时，它不返回`robjects`实例。我们正在摆脱它，因为像[`int`](http://docs.python.org/library/functions.html#int "(in Python v2.7)")、[`float`](http://docs.python.org/library/functions.html#float "(in Python v2.7)")、[`bool`](http://docs.python.org/library/functions.html#bool "(in Python v2.7)")、[`complex`](http://docs.python.org/library/functions.html#complex "(in Python v2.7)")、[`str`](http://docs.python.org/library/functions.html#str "(in Python v2.7)")这样的原子 python 类型在`rinterface`/c 级别被 rpy2 很好地处理。