# 交互工作

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/interactive.html](https://rpy2.readthedocs.io/en/version_2.8.x/interactive.html)

## 概述

注意

这是一个实验包，这里实验的一些想法已经实现了。

对于交互工作，“r magic”扩展到`ipython`是交互工作的首选方式/最受测试的方式。

## ipython 魔法集成（was rmagic）

## R 事件循环

为了执行刷新交互式图形设备之类的操作，R 需要处理触发刷新的事件。

```
>>> from rpy2.interactive import process_revents
>>> process_revents.start()

```

```
>>> from rpy2.robjects.packages import importr
>>> from rpy2.robjects.vectors import IntVector
>>> graphics = importr("graphics")
>>> graphics.barplot(IntVector((1,3,2,5,4)), ylab="Value")

```

现在 R 图形设备在调整大小时会更新。如果希望停止处理事件：

```
>>> process_revents.stop()

```

处理过程可以恢复，再次停止，并且可以随意重复 _ad-libitum_。

可以粗略地控制执行 R 事件处理的频率。在每次处理 R 事件之间，线程将休眠一段任意时间。

```
>>> process_revents.EventProcessor.interval
0.2

```

如果希望进行更频繁的处理，可以更改此值并将其设置为其他值。这可以在线程处理处于活动状态时完成，并将在下一个睡眠周期考虑在内。

```
>>> process_revents.EventProcessor.interval = 1.0

```

## 交互式工作的实用程序

Note

这个模块包含了许多实验特性，其中一些已经不再需要了，因为“r 魔法”扩展了 ipython。它们正在被弃用，并将在将来的版本中从代码库中删除。

R 经常被用作交互顶层，或者读 eval 打印循环（repl）。这在分析数据时很方便：键入一些代码，得到结果，键入一些新代码，并根据结果进一步分析。

python 也可以以类似的方式使用，但是默认 python 控制台的局限性导致了替代控制台和交互式开发编辑器的创建（idle、ipython、bpython、emacs 模式、komodo…）。诸如代码突出显示、自动完成和帮助字符串或函数签名的方便显示等特性使这些工具变得很有价值。

包[`rpy2.interactive`](#module-rpy2.interactive "rpy2.interactive")针对交互式用户，但也可用于非交互式代码。它是交易的灵活性或性能，以便于使用。

```
>>> import rpy2.interactive as r
>>> import rpy2.interactive.packages # this can take few seconds
>>> v = r.IntVector((1,2,3))
>>> r.packages.importr('datasets')
rpy2.robjecs.packages.Package as a <module 'datasets' (built-in)>
>>> data = rpy2.interactive.packages.data
>>> rpackages = r.packages.packages
>>> # list of datasets
>>> data(rpackages.datasets).names()
# list here
>>> env = data(rpackages.datasets).fetch('trees')
>>> tuple(env['trees'].names)
('Girth', 'Height', 'Volume')

```

### R 向量

```
>>> import rpy2.interactive as r
>>> r.IntVector(range(10))
<IntVector - Python:0x935774c / R:0xa22b440>
[       0,        1,        2, ...,        7,        8,        9]
>>> r.IntVector(range(100))
<IntVector - Python:0xa1c2dcc / R:0x9ac5540>
[       0,        1,        2, ...,       97,       98,       99]

```

在 R 中，没有标量。

```
>>> r.packages.base.pi
<FloatVector - Python:0xa1d7a0c / R:0x9de02a8>
[3.141593]

```

要了解更多信息，请检查[R 矢量](introduction.html#introduction-vectors)部分。

### R 包

R 拥有丰富的软件包选择，在其他计算机语言和系统中称为库。

R 包可以是：

*   在 R 存储库中可用（公共或私有）
*   安装
*   附加（加载）

#### 正在加载已安装的包

启动 r 时，将加载 _ 基 _ 包以及默认情况下的包 _grdevices_、_graphics_、_methods_、_stats_ 和 _utils_。

我们从加载 R 包开始，因为这是 R 中非常常见的操作，而且由于 R 通常与 _ 推荐的 _ 包一起分发，所以可以立即开始使用。

可以通过函数`importr()`加载已安装的 R 包。

```
>>> import rpy2.interactive as r
>>> import rpy2.interactive.packages # this can take few seconds
>>> r.packages.importr("cluster")
rpy2.robjecs.packages.Package as a <module 'cluster' (built-in)>

```

函数返回一个 package 对象，并在`r.packages.packages`中添加对它的引用。

```
>>> rlib = r.packages.packages
>>> rlib.cluster
rpy2.robjecs.packages.Package as a <module 'cluster' (built-in)>

```

随后可以通过该命名空间对象访问 R 包 _ 集群 _ 中的所有对象。例如，对于函数条形图：

```
>>> rlib.cluster.silhouette
<SignatureTranslatedFunction - Python:0x24f9418 / R:0x2f5b008>

```

同样，可以加载其他包，如 _nlme_ 和 _ 数据集 _。

```
>>> r.packages.importr("nlme")
rpy2.robjecs.packages.Package as a <module 'stats' (built-in)>
>>> r.packages.importr("datasets")
rpy2.robjecs.packages.Package as a <module 'datasets' (built-in)>

```

然后，我们可以通过一个图形示例演示如何访问 R 包中的对象：

```
r.packages.graphics.coplot(r.Formula('Time ~ conc | Wt'),
                           r.packages.datasets.Theoph)

```

#### 可用程序包

R 具有列出可用包的功能。

```
>>> import rpy2.interactive as r
>>> import rpy2.interactive.packages # this can take few seconds
>>> r.packages.importr("utils")
>>> rlib = r.packages.packages
>>> m = rlib.utils.available_packages()

```

返回的对象是一个`rpy2.robjects.vectors.Matrix`，每行一个包（默认的 CRAN 存储库中有许多包）。

```
>>> tuple(m.dim)
(2692, 13)

```

```
>>> tuple(m.colnames)
('Package', 'Version', 'Priority', 'Depends', 'Imports', 'LinkingTo', 'Suggests', 'Enhances', 'OS_type', 'License', 'Archs', 'File', 'Repository')

```

Note

可以指定特定的存储库。

例如，使用生物导体。

```
import rpy2.rinteractive as r

bioc_rooturl = "http://www.bioconductor.org/packages"
bioc_version = "2.7"
bioc_sections = ("bioc", "data/annotation", "data/experiment", "extra")

repos = r.vectors.StrVector(["/".join((bioc_rooturl, bioc_version, x)) for x in bioc_sections])

m_bioc = rlib.utils.available_packages(contriburl = r.packages.utils.contrib_url(repos))

```

#### 安装程序包

Note

要为存储库安装包，我们必须加载包`utils`。有关加载包的详细信息，请参阅加载包一节。

```
>>> import rpy2.interactive as r
>>> import rpy2.interactive.packages # this can take few seconds
>>> rlib = r.packages.packages
>>> r.packages.importr("utils")
>>> package_name = "lme4"
>>> rlib.utils.install_packages(package_name)

```

一旦安装了一个包，它就可以供将来使用，而无需再次安装（除非使用了不同版本的 R）。