# NumPy

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/numpy.html](https://rpy2.readthedocs.io/en/version_2.8.x/numpy.html)

使用 python 进行科学计算的流行解决方案是`numpy`（以前的实例是`Numpy`和`numarray`）。

`rpy2`具有便于与`numpy`进行双向通信的功能。

## 高级接口

### 从`rpy2`到`numpy`：

r 向量或数组可以使用`numpy.array()`或`numpy.asarray()`转换为`numpy`数组：

```
import numpy

ltr = robjects.r.letters
ltr_np = numpy.array(ltr)

```

此行为继承自低级接口；继承自`rpy2.rinterface.SexpVector`的类向量对象表示由`numpy`识别的接口。

```
from rpy2.robjects.packages import importr, data
import numpy

datasets = importr('datasets')
ostatus = data(datasets).fetch('occupationalStatus')['occupationalStatus']
ostatus_np = numpy.array(ostatus)
ostatus_npnc = numpy.asarray(ostatus)

```

矩阵 _ostatus_ 是 8x8 矩阵：

```
>>> print(ostatus)
      destination
origin   1   2   3   4   5   6   7   8
     1  50  19  26   8   7  11   6   2
     2  16  40  34  18  11  20   8   3
     3  12  35  65  66  35  88  23  21
     4  11  20  58 110  40 183  64  32
     5   2   8  12  23  25  46  28  12
     6  12  28 102 162  90 554 230 177
     7   0   6  19  40  21 158 143  71
     8   0   3  14  32  15 126  91 106

```

其内容已复制到 numpy 数组：

```
>>> ostatus_np
array([[ 50,  19,  26,   8,   7,  11,   6,   2],
       [ 16,  40,  34,  18,  11,  20,   8,   3],
       [ 12,  35,  65,  66,  35,  88,  23,  21],
       [ 11,  20,  58, 110,  40, 183,  64,  32],
       [  2,   8,  12,  23,  25,  46,  28,  12],
       [ 12,  28, 102, 162,  90, 554, 230, 177],
       [  0,   6,  19,  40,  21, 158, 143,  71],
       [  0,   3,  14,  32,  15, 126,  91, 106]])
>>> ostatus_np[0, 0]
50
>>> ostatus_np[0, 0] = 123
>>> ostatus_np[0, 0]
123
>>> ostatus.rx(1, 1)[0]
50

```

另一方面，_ostatus_npnc_ 是对 _ostatus_ 的视图；未复制：

```
>>> ostatus_npnc[0, 0] = 456
>>> ostatus.rx(1, 1)[0]
456

```

因为我们确实修改了会话的实际 R 数据集，所以我们应该还原它：

```
>>> ostatus_npnc[0, 0] = 50

```

如我们所见，`numpy.asarray()`：提供了一种在底层 R 数组上构建 _ 视图 _ 的方法，而不进行复制。这对于开发人员特别有吸引力，他们需要将`rpy2`和`numpy`代码与`rpy2`对象或`numpy`视图混合到函数中，或者对于更熟悉`numpy`语法的交互用户。

注意

当前接口依赖于 numpy 中定义的 _uuu array_struct_uuug1]。_

python 缓冲区，如[**pep 3118**](https://www.python.org/dev/peps/pep-3118)中所定义，是通往未来的道路，而 rpy2 已经提供了它们……尽管作为一个（记录不充分的）实验特征。

### 从`numpy`到`rpy2`：

将`numpy`对象自动转换为`rpy2`对象的激活（和停用）可以使用以下方法进行：

```
from rpy2.robjects import numpy2ri
numpy2ri.activate()
numpy2ri.deactivate()

```

警告

在早期版本的 RPY2 中，只有导入才需要进行转换。导入模块时的副作用可能会导致问题，现在有一个额外的步骤来激活转换：调用函数`rpy2.robjects.numpy2ri.activate()`。

Note

为什么要将其作为可选导入，而它可能已包含在函数`py2ri()`中（与为该函数提交的原始补丁中所做的相同）？

虽然这两个选项都是有效和合理的，但是设计决策是为了将`rpy2`与`numpy`最大程度地分离，并且不假定自动安装`numpy`意味着程序员想要使用它。

Note

模块`numpy2ri`是一个示例，说明如何执行与[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")之间的自定义转换。

## 低电平接口

`rpy2.rinterface.SexpVector`对象的行为与 python 包`numpy`中定义的数组类似。

函数`numpy.array()`和`numpy.asarray()`可用于构造`numpy`数组：

```
>>> import numpy
>>> rx = rinterface.SexpVector([1,2,3,4], rinterface.INTSXP)
>>> nx = numpy.array(rx)
>>> nx_nc = numpy.asarray(rx)

```

Note

使用`numpy.asarray()`时，不会复制数据。

```
>>> rx[2]
3
>>> nx_nc[2] = 42
>>> rx[2]
42
>>>

```