
# 低级接口

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/rinterface.html](https://rpy2.readthedocs.io/en/version_2.8.x/rinterface.html)

## 概述

包`rinterface`作为较低级别的接口提供，用于`robjects`处理的用例未被覆盖的情况，或用于`robjects`中的层在性能方面成本过高的情况。

该包可以通过以下方式导入：

```
>>> import rpy2.rinterface as rinterface

```

### 初始化

在做很多事情之前，必须先初始化 r。函数`initr()`用于初始化嵌入的 r。

这是通过函数`initr()`完成的。

```
>>> rinterface.initr()

```

初始化只能执行一次。为了避免在使用嵌入式 R 时出现不可预知的结果，对`initr()`的后续调用将不会有任何效果。

函数`get_initoptions()`和`set_initoptions()`可用于修改选项。初始化的默认参数在模块变量`initoptions`中。

警告

当前，默认初始化选项集包含`–vanilla`，这意味着无论何时设置，都会忽略`R_LIBS`。在启动**r**之前，必须设置初始化选项。

这可以非常简单地通过在初始化嵌入式 R 之前添加以下行来实现，或者在执行初始化时导入[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")。

```
import rpy2.rinterface
rpy2.rinterface.set_initoptions(('rpy2', '--verbose', '--no-save'))

```

注意

如果调用`initr()`返回一个错误，说明未定义`R_HOME`，则应该在路径中有**r**可执行文件（在 Unix Alikes 上有`PATH`可执行文件，或者在 Microsoft Windows 上有`Path`可执行文件），或者定义了环境变量`R_HOME`。

如果初始化失败，则应调查针对 R RPY2 的版本与运行 R RPY2 的版本之间的不匹配。变量`rpy2.rinterface.R_BUILD_VERSION`包含有关 r 版本 rpy2 的信息。RPY2 相对独立于 R 版本，但是 R C API 中的更改可能会导致问题。

#### 结束 R

结束 R 进程是可能的，但是用`initr()`再次启动它确实会导致一个几乎不可用的 R 进程。出于这个原因，如果可以的话，应该仔细考虑使用`endr()`。

Note

在为 r 编写 GUI 时，开发人员可能希望阻止用户调用**r**`quit()`，或者确保在终止 r 之前执行特定代码（例如，确认对话框窗口“是否确实要终止？”“）。这可以通过用适当的函数替换回调`cleanup`来实现（请参见[清理](callbacks.html#rinterface-callbacks-cleanup)）。

### r 空间和 python 空间

使用 rpy2 包时，有两个领域共存：r 和 python。

可以将`Sexp_Type`对象视为指向存储和管理在 R 空间中的数据的 Python 信封。

R 变量存在于嵌入式 R 工作空间中，可以通过它们的 Python 对象表示从 Python 访问。

我们区分了两种 R 对象：命名对象和匿名对象。命名对象在 R 工作区中有一个关联的符号。

#### 命名对象

例如，下面的 r 代码正在`global environment`中分别创建两个名为`x`和`hyp`的对象。这两个对象可以使用它们的名称从 Python 访问。

```
x <- c(1,2,3)

hyp <- function(x, y) sqrt(x^2 + y^2)

```

提供两种环境作为`rpy2.rinterface.SexpEnvironment`

环球女星

全局环境可以看作是根（或最顶层）环境，实际上是一个环境的列表，它是一个环境序列。

当一个 R 库（用 R 术语表示的包）被加载时，它是否被添加到现有的环境序列中。除非另有规定，否则插入第二个位置。第一个位置总是归因于全球环境。该库被称为附加到当前搜索路径。

基环境

基本包有一个命名空间，可以作为环境访问。

Note

根据`globalenv`中的内容和附加的包，从`globalenv`开始搜索时可以屏蔽基对象。当您希望确保访问您知道位于基命名空间中的函数时，请使用`baseenv`。

#### 匿名对象

匿名 R 对象没有关联的符号，但受到垃圾收集的保护。

当使用`Sexp*`类的构造函数时，可以创建此类对象。

例如：

```
>>> x = rinterface.IntVector((1,2,3))

```

创建一个完全可用的 r 向量，但它没有关联的 r 符号（它在内存中，但不能用名字 fomr r 调用）。在删除`x`和 python 垃圾收集器销毁`x`之前，它还受到垃圾收集的保护。

Note

要完成所用内存的恢复，还必须调用 R 垃圾收集器。当达到内存使用的阈值时，运行 R 代码时应该自动发生这种情况，但也可以显式地调用这两个垃圾收集器。有关详细信息，请参阅[内存管理和垃圾收集](rinterface-memorymanagement.html#rinterface-memory)。

### 传递价值范式

R 语言的根是函数型的，参数由值传递。R 实际上使用一些技巧来降低内存使用率，例如只在需要时复制对象（即在本地块中修改对象时），但对象的复制仍然很频繁。在使用大型对象或执行大量对象修改之前，用户可能不会注意到这一点，在这种情况下，可能会出现性能问题。一个臭名昭著的例子是当一个矩阵的列名被改变时，当一个矩阵非常大时，整个矩阵最终被复制时，会使系统陷入瘫痪。

相反，python 使用的是通过函数调用传递的指针对象，由于`rpy2`是一个 python-to-r 接口，所以 python 方法是保守的。

尽管是人为的，下面的例子将说明这一点。使用 r，重命名列如下：

```
# create a matrix
m <- matrix(1:10, nrow = 2,
            dimnames = list(c("1", "2"),
                            c("a", "b", "c", "d", "e")))
# rename the third column
i <- 3
colnames(m)[i] <- "foo"

```

带[`rpy2.rinterface`](#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")：

```
# import and initialize
import rpy2.rinterface as ri
ri.initr()

# make a function to rename column i
def rename_col_i(m, i, name):
    m.do_slot("dimnames")[1][i] = name

# create a matrix
matrix = ri.baseenv["matrix"]
rlist = ri.baseenv["list"]
m = matrix(ri.baseenv[":"](1, 10),
           nrow = 2,
           dimnames = rlist(ri.StrSexpVector(("1", "2")),
                            ri.StrSexpVector(("a", "b", "c", "d", "e"))))

```

现在我们可以检查列名称

```
>>> tuple(m.do_slot("dimnames")[1])
('a', 'b', 'c', 'd', 'e')

```

并重命名第三列（记住 r 向量是 1 索引的，而 python 序列是 0 索引的）。

```
>>> i = 3-1
>>> rename_col_i(m, i, ri.StrSexpVector(("foo", )))
>>> tuple(m.do_slot("dimnames")[1])
('a', 'b', 'foo', 'd', 'e')

```

与 R 代码不同，矩阵或具有列名的向量都不会被复制。当这不是一件好事时，可以像通常复制 python 对象那样复制 r 对象（使用[`copy.deepcopy()`](http://docs.python.org/library/copy.html#copy.deepcopy "(in Python v2.7)")，[`Sexp`](#rpy2.rinterface.Sexp "rpy2.rinterface.Sexp")实现[`Sexp.__deepcopy__()`](#rpy2.rinterface.Sexp.__deepcopy__ "rpy2.rinterface.Sexp.__deepcopy__")）。

### 解析和评估 R 代码

用于解析任意字符串 r 代码的 r c 级函数作为函数`parse()`公开。

```
>>> expression = ri.parse('1 + 2')

```

结果表达式是一个嵌套的 r 语句列表。

```
>>> len(expression)
1
>>> len(expression[0])
3

```

r 代码 _1+2_ 转换为长度为 3 的表达式：_+（1，2）_，这是对带有参数 _1_ 和 _2_ 的函数 _+_（或者更确切地说与函数关联的符号）的调用。

```
>>> ri.str_typeint(expression[0][0].typeof)
'SYMSXP'
>>> tuple(expression[0][1])
(1.0,)
>>> tuple(expression[0][2])
(2.0,)

```

Note

如果需要执行结果，则必须计算表达式。

### 从 r 调用 python 函数

正如从 R 的函数根可以预期的那样，函数是第一类对象。这意味着使用回调函数作为参数传递的情况并不少见，这也意味着 Python 程序员必须能够为函数编写 r 代码作为参数，或者能够将 python 函数作为真正的 r 函数传递给 r。最后一个选项正在成为可能，换句话说，我们可以编写一个 python 函数，并以嵌入式 R 引擎可以用作常规 R 函数的方式将它暴露给 R。

作为一个例子，让我们考虑 r 函数 _optim（）_，它为给定的成本函数寻找最佳参数。调用 _optim（）_ 时应传递 cost 函数，因为在探索参数空间时将重复调用该函数，并且只有 python 编码技能是必需的，正如下面的代码所示。

```
from rpy2.robjects.vectors import FloatVector
from rpy2.robjects.packages import importr
import rpy2.rinterface as ri
stats = importr('stats')

# cost function, callable from R
@ri.rternalize
def cost_f(x):
    # Rosenbrock Banana function as a cost function
    # (as in the R man page for optim())
    x1, x2 = x
    return 100 * (x2 - x1 * x1)**2 + (1 - x1)**2

# starting parameters
start_params = FloatVector((-1.2, 1))

# call R's optim() with our cost funtion
res = stats.optim(start_params, cost_f)

```

为了方便起见，代码示例尽可能使用高级接口对象。

低级函数`rternalize()`将接受一个任意的 python 函数并返回一个`rinterface.SexpClosure`实例，这是一个 R 可以用作函数的对象。

## 交互式功能

从`rpy2`启动的嵌入式 R 默认为交互式，这意味着在交互式 R 控制台中工作时存在的许多交互式功能将可供使用。

这些功能可以由`rpy2`用户显式调用，但也可以间接触发，因为与在所谓的 _ 批处理模式 _ 下运行相比，当以交互方式运行时，r 函数上的某些功能的行为会有所不同。

Note

但是，交互式使用可能意味着能够定期检查和处理事件。例如，对于交互式图形设备或基于 HTML 的帮助系统（请参见[处理交互式事件](#rinterface-interactive-processevents)）。

### 带 R 控制台的 I/O

参见[控制台 I/O](callbacks.html#rinterface-callbacks-consoleio)。

### 处理交互式事件

交互式 R 会话可以启动需要持续监视事件的交互式活动。一个典型的例子是交互式图形设备（`plotting windows`），因为它们可以调整大小并刷新显示的信息。

但是，要这样做，必须指示 r 进程处理挂起的交互事件。例如，这是由 R 控制台完成的，但是`rpy2`被设计为一个库，而不是在 python 中运行的线程 R 进程（但是可以这样做，如下所示）。

恢复交互性的方法就是定期调用函数`rinterface.process_revents()`。

提供了一个更高级的接口，运行线程中 R 事件的处理（参见[R 事件循环](interactive.html#interactive-reventloop)一节）。

## 类

### [`Sexp`](#rpy2.rinterface.Sexp "rpy2.rinterface.Sexp")

类[`Sexp`](#rpy2.rinterface.Sexp "rpy2.rinterface.Sexp")是所有 R 对象的基类。

_class_ `rpy2.rinterface.``Sexp`

`__sexp__`

python c capsule 包装指向底层 r 对象的指针（`SEXP`）

`named`

**r**不计算其对象的引用数。此方法返回`NAMED`值（整数）。有关更多详细信息，请参阅 R 扩展手册。

`typeof`

基础 r 对象的内部 r 类型

```
&gt;&gt;&gt; letters.typeof
16

```

`__deepcopy__`(_self_)

复制对象的 _deep_ 副本，调用 r-api c 函数 c:函数：：`Rf_duplicate()`以复制包装好的 r 对象。

2.0.3 版新增。

`do_slot`(_name_)

r 对象可以被赋予属性。在 r 中，函数 _attr_ 允许一个人访问一个对象的属性；它在 r 的 C 接口中被称为[`do_slot()`](#rpy2.rinterface.Sexp.do_slot "rpy2.rinterface.Sexp.do_slot")。

&lt;colgroup&gt;&lt;col class="field-name"&gt; &lt;col class="field-body"&gt;&lt;/colgroup&gt;
| 参数： | **名称**–字符串 |
| --- | --- |
| 返回类型： | [`Sexp`](#rpy2.rinterface.Sexp "rpy2.rinterface.Sexp")的实例 |
| --- | --- |

```
&gt;&gt;&gt; matrix = rinterface.globalenv.get("matrix")
&gt;&gt;&gt; letters = rinterface.globalenv.get("letters")
&gt;&gt;&gt; m = matrix(letters, ncol = 2)
&gt;&gt;&gt; [x for x in m.do_slot("dim")]
[13, 2]
&gt;&gt;&gt;

```

`do_slot_assign`(_name_, _value_)

为具有给定名称的插槽分配值，在不存在插槽的情况下创建插槽。

&lt;colgroup&gt;&lt;col class="field-name"&gt; &lt;col class="field-body"&gt;&lt;/colgroup&gt;
| Parameters: | 

*   **名称**–字符串
*   **值**–实例[`Sexp`](#rpy2.rinterface.Sexp "rpy2.rinterface.Sexp")

 |
| --- | --- |

`rsame`(_sexp_obj_)

告诉 sexp 对象的底层 r 对象是否相同。

&lt;colgroup&gt;&lt;col class="field-name"&gt; &lt;col class="field-body"&gt;&lt;/colgroup&gt;
| Return type: | 布尔值 |
| --- | --- |

#### 底层 R 对象

底层 r 对象是指向 r 的`Rinternals.h`中定义的 c:type:：`SEXPREC`的指针。该对象被包装在 c:type:：`SexpObj`中，并放置在 python 胶囊中。

胶囊提供了一种相对安全的机制来在 rpy2 对象之间交换底层的 r 对象。

```
from rpy2.rinterface import SexpIntVector, SexpFloatVector
vector1=SexpIntVector((1, 2, 3))
vector2=SexpFloatVector((4.0, 5.0, 6.0))

vector1.__sexp__ = vector2.__sexp_

```

### `SexpVector`

#### 概述

在 R 中，所有的标量实际上都是向量。任何一个值变量都是长度为 1 的向量。

再次使用常量 _pi_：

```
>>> pi = rinterface.globalenv.get("pi")
>>> len(pi)
1
>>> pi
<rinterface.SexpVector - Python:0x2b20325d2660 / R:0x16d5248>
>>> pi[0]
3.1415926535897931
>>>

```

（西方）字母表的字母是：

```
>>> letters = rinterface.globalenv.get("letters")
>>> len(letters)
26
>>> LETTERS = rinterface.globalenv.get("LETTERS")

```

#### R 型

R 向量都有一个`type`，有时在 R 中称为`mode`。这个信息被 R 编码为一个整数，但有时人类读者能够提供一个字符串会更好。

`rpy2.rinterface.``str_typeint`(_typeint_)

返回与整数编码的 r 类型相对应的字符串。

&lt;colgroup&gt;&lt;col class="field-name"&gt; &lt;col class="field-body"&gt;&lt;/colgroup&gt;
| Parameters: | **typeint**–整数（由[`Sexp.typeof`](#rpy2.rinterface.Sexp.typeof "rpy2.rinterface.Sexp.typeof")返回） |
| --- | --- |
| Return type: | [字符串](http://docs.python.org/library/string.html#module-string "(in Python v2.7)") |
| --- | --- |

#### 索引

索引的工作方式与对常规`Python`元组或列表的工作方式相同。索引从 0（零）开始，这与**R**不同，其中索引从 1（一）开始。

Note

_uugetitem_uuu_ 运算符 _[_ 正在返回 python 标量。将一个 _sexpvector_ 强制转换到一个列表中只需要迭代它，或者简单地调用构造函数`list()`。

#### 公共属性

姓名

在 r 中，向量可以被命名，也就是说，向量中的每个值都可以被命名（与字符串关联）。这些名称作为一个属性添加到另一个名称中（方便地称为`names`），可以这样访问：

```
>>> options = rinterface.globalenv.get("options")()
>>> option_names = options.do_slot("names")
>>> [x for x in options_names]

```

Note

名称向量中的元素不必是唯一的。python 对应项作为`rpy2.rlike.container.TaggedList`提供。

尺寸和尺寸名称

对于`array`，可以通过名为`dimnames`的槽访问对象各个维度上的名称。

缺少值

Note

R 还具有函数调用中缺少参数的概念。这是一个单独的概念，有关的更多信息，请参见[函数](#rinterface-functions)一节。

在 r 中，缺少符号 _na_ 表示缺少值。R 标量实际上是向量的一般规则在这里再次适用，下面的 R 代码创建一个长度为 1 的向量。

```
x <- NA

```

NA 的类型是逻辑的（布尔型），可以用符号 _NA_character_uug1]、_NA_integer_ug3]、_NA_real_uu5]和 _NA_complex_7]指定不同的类型。____

在[`rpy2.rinterface`](#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")中，可以通过`NA_Character`、`NA_Integer`、`NA_Real`访问符号。

这些是来自相应 _na&lt；something&gt；类型 _ 类的单例实例。

```
>>> my_naint = rinterface.NAIntegerType()
>>> my_naint is rinterface.NA_Integer
True
>>> my_naint == rinterface.NA_Integer
True

```

_na_ 值可以存在于 r 函数返回的向量中。

```
>>> rinterface.baseenv['as.integer'](rinterface.StrSexpVector(("foo",)))[0]
NA_integer_

```

_na_ 值可以实现运算符，但结果将丢失值。

```
>>> rinterface.NA_Integer + 1
NA_integer_
>>> rinterface.NA_Integer * 10
NA_integer_

```

Warning

依赖 C 级实现的 python 函数可能不遵循 _nas_ 的相同规则。

```
>>> x = rinterface.IntSexpVector((1, rinterface.NA_Integer, 2))
>>> sum(x)
3
>>> max(x)
2
>>> min(x)
NA_integer_

```

这应该是使用 r's na 的首选方法，因为这些符号很少有特殊性，不能用`SexpEnvironment.get()`检索。

这些缺少的值也可以与[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")层一起使用，在那里可以找到更多关于它们用法的文档（请参见[缺少的值](vector.html#robjects-missingvalues)）。

施工人员

提供了便利类来创建给定类型的向量：

### `SexpEnvironment`

#### `__getitem__()`/`__setitem__()`/`__delitem__()`

运算符只在环境中查找符号，而不在封闭环境的路径中进一步查找。

以下将返回异常`LookupError`：

```
>>> rinterface.globalenv["pi"]

```

常量 _pi_ 是在 r 的 _base_ 包中定义的，因此在全局环境中找不到。

为环境中的符号赋值与为 Python 字典中的键赋值一样简单：

```
>>> x = rinterface.IntSexpVector([123, ])
>>> rinterface.globalenv["x"] = x
>>> len(x)
1
>>> tuple(rinterface.globalenv)
('x', )

```

删除元素可以像对 python[`dict`](http://docs.python.org/library/stdtypes.html#dict "(in Python v2.7)")那样完成：

```
>>> del(rinterface.globalenv['x'])
>>> len(x)
0

```

Note

并非所有的 R 环境都是哈希表，这可能会影响执行重复查找时的性能。

Note

在 R 空间中复制 R 对象。

#### `__iter__()`

这个物体是可以转动的。

例如，我们使用基名称空间（即包含 r 的基对象的环境：

```
>>> base = rinterface.baseenv
>>> basetypes = [x.typeof for x in base]

```

Warning

在当前的实现中，当创建迭代器时，环境的内容只评估一次。向环境中添加或删除元素不会更新迭代器（这是一个问题，将在不久的将来得到解决）。

#### `get()`

每当执行符号搜索时，都会考虑整个搜索路径：依次检查列表中的环境，并返回找到匹配的第一个符号的值。

让我们从一个例子开始：

```
>>> rinterface.globalenv.get("pi")[0]
3.1415926535897931

```

常量`pi`在包`base`中定义，该包始终位于搜索路径中（并位于最后一个位置，因为它首先连接）。对`get()`的调用将首先在`globalenv`中查找`pi`，然后在搜索路径中的下一个环境中查找`pi`并重复此操作，直到找到对象或要搜索的环境序列耗尽。

我们知道`pi`在基名称空间中，我们可以直接从那里得到：

```
>>> ri.baseenv.get("pi")[0]
3.1415926535897931
>>> ri.baseenv["pi"][0]
3.1415926535897931
>>> ri.globalenv["pi"][0]
Traceback (most recent call last):
  File "<stdin>", line 1, in <module>
LookupError: 'pi' not found

```

**r**可以专门查找函数，这是在对已分析的函数调用进行计算时发生的。下面的**r**交互式会话示例应该演示它：

```
> mydate <- "hohoho"
> mydate()
Error: could not find function "mydate"
>
> date <- "hohoho"
> date()
[1] "Sat Aug  9 15:27:40 2008"

```

基本函数`date`仍然存在，尽管在搜索路径的前面有一个非函数对象。

使用可选参数`wantfun`可以从`rpy2`获得相同的行为（指定`get()`应返回一个 r 函数）。

```
>>> ri.globalenv["date"] = ri.StrSexpVector(["hohoho", ])
>>> ri.globalenv.get("date")[0]
'hohoho'
>>> ri.globalenv.get("date", wantfun=True)
<rinterface.SexpClosure - Python:0x7f142aa96198 / R:0x16e9500>
>>> date = ri.globalenv.get("date", wantfun=True)
>>> date()[0]
'Sat Aug  9 15:48:42 2008'

```

#### R 包作为环境

从程序员的角度来看，将加载的**r**包映射为模块并提供对包中**r**对象的访问，这与访问模块中`Python`对象的方式相同。

不幸的是，这在一个完全健壮的方式中是不可能的：点字符`.`可以用于 r 中的符号名（几乎与任何字符一样），这使得**r**和`Python`名称之间的精确对应相当困难。`rpy`使用转换函数将“.”转换为“u”和“back”，但这可能会导致复杂的情况，因为“u”也可以用于 r 符号（尽管这是高级接口所采用的方法，请参见[r packages](robjects_rpackages.html#robjects-packages)一节）。

有一种方法可以明确地访问 R 包中的对象，因为加载的包可以被视为环境。为了方便使用，可以考虑使用如下函数：

```
def rimport(packname):
    """ import an R package and return its environment """
    as_environment = rinterface.baseenv['as.environment']
    require = rinterface.baseenv['require']
    require(rinterface.StrSexpVector(packname),
            quiet = rinterface.BoolSexpVector((True, )))
    packname = rinterface.StrSexpVector(('package:' + str(packname)))
    pack_env = as_environment(packname)
    return pack_env

```

```
>>> class_env = rimport("class")
>>> class_env['knn']

```

例如，我们可以在`Python`中重新实现**r**函数，返回搜索路径（`search`）。

```
def rsearch():
    """ Return a list of package environments corresponding to the
    R search path. """
    spath = [ri.globalenv, ]
    item = ri.globalenv.enclos()
    while not item.rsame(ri.emptyenv):
        spath.append(item)
        item = item.enclos()
    spath.append(ri.baseenv)
    return spath

```

作为另一个示例，可以简单地实现一个函数，该函数返回由`get()`调用的对象来自哪个环境。

```
def wherefrom(name, startenv=ri.globalenv):
    """ when calling 'get', where the R object is coming from. """
    env = startenv
    obj = None
    retry = True
    while retry:
        try:
            obj = env[name]
            retry = False
        except LookupError, knf:
            env = env.enclos()
            if env.rsame(ri.emptyenv):
                retry = False
            else:
                retry = True
    return env

```

```
>>> wherefrom('plot').do_slot('name')[0]
'package:graphics'
>>> wherefrom('help').do_slot('name')[0]
'package:utils'

```

Note

不幸的是，这并不能概括为所有情况：基包没有名称。

```
>>> wherefrom('get').do_slot('name')[0]
Traceback (most recent call last):
File "<stdin>", line 1, in <module>
LookupError: The object has no such attribute.

```

### 功能

具有上下文的函数

在 R 术语中，闭包是一个函数（具有封闭环境）。可以将封闭环境视为函数的上下文。

Note

从技术上讲，类`SexpClosure`对应于 r 类型 closxp、builtinsxp 和 specialsxp，只有第一个（closxp）是一个闭包。

```
>>> sum = rinterface.globalenv.get("sum")
>>> x = rinterface.IntSexpVector([1,2,3])
>>> s = sum(x)
>>> s[0]
6

```

命名参数

R 函数的命名参数可以按照其他任何常规 python 函数的方式指定。

```
>>> rnorm = rinterface.globalenv.get("rnorm")
>>> rnorm(rinterface.IntSexpVector([1, ]),
          mean = rinterface.IntSexpVector([2, ]))[0]
0.32796768001636134

```

但是，R 参数的名称经常会引起问题：所有带有 _ 点 _ 的名称。为 R 函数使用这些参数将需要：

*   在带有命名参数的字典上使用特殊语法`**kwargs`
*   使用方法`rcall()`。

命名参数的顺序

r 中的函数调用与 python 中的函数调用不同的一点是，r 中的所有参数都是按调用中的顺序传递的（无论参数是否命名），而在 python 中，只有没有名称的参数是按顺序传递的。使用模块[`rpy2.rlike.container`](rlike.html#module-rpy2.rlike.container "rpy2.rlike.container")中的类`OrdDict`和方法`rcall()`允许以与在 r 中相同的方式调用函数。例如：

```
import rpy2.rlike.container as rpc
args = rpc.OrdDict()
args['x'] = rinterface.IntSexpVector([1,2,3])
args[None] = rinterface.IntSexpVector([4,5])
args['y'] = rinterface.IntSexpVector([6, ])
rlist = rinterface.baseenv['list']
rl = rlist.rcall(args.items(), rinterface.globalenv)

```

```
>>> [x for x in rl.do_slot("names")]
['x', '', 'y']

```

封闭环境

在下面的示例中，我们检查函数 _plot_ 的环境，该函数是包 _ 图形 _ 的命名空间。

```
>>> plot = rinterface.globalenv.get("plot")
>>> ls = rinterface.globalenv.get("ls")
>>> envplot_list = ls(plot.closureEnv())
>>> [x for x in envplot_ls]
>>>

```

缺少参数

In R 函数调用可以包含明显缺少的参数。

```
> sum(1,,3)
Error: element 2 is empty;
   the part of the args list of 'sum' being evaluated was:
   (1, , 3)

```

这在提取数组的子集时使用，提取函数`[`解释了缺少的参数，就像必须提取该维度中的所有元素一样。

```
m <- matrix(1:10, nrow = 5, ncol = 2)

# extract the second column
n <- m[, 2]

# can also be written
n <- "["(m, , 2)

```

`rinterface.MissingArg`是指向 singleton`rinterface.MissingArgType`的指针，允许显式地将缺少的参数传递给函数调用。

例如，提取上面所示的带有 r 的矩阵的第二列，将在 rpy2 中写得几乎相同。

```
import rpy2.rinterface as ri
ri.initr()

matrix = ri.baseenv['matrix']
extract = ri.baseenv['[']

m = matrix(ri.IntSexpVector(range(1, 11)), nrow = 5, ncol = 2)

n = extract(m, ri.MissingArg, 2)

```

### `SexpS4`

R 中的面向对象编程有几种风格，其中一种叫做`S4`。它在 r 的 c-api 级别上有自己的类型，因此我们定义了一个类。除此之外，类不提供太多特定的特性（请参见下面的类的 pydoc）。

实例的属性可以通过父类[`Sexp`](#rpy2.rinterface.Sexp "rpy2.rinterface.Sexp")方法`do_slot()`访问。

### `SexpExtPtr`

外部指针是为了便于处理来自 R 的 C 或 C++数据结构，换句话说，它们是指向 _ 外部 _ 到 R 的指针。它们已经被用来在共享内存中实现向量和数组，或者基于存储的向量和数组。

外部指针也不遵守传递值规则，并且可以表示在 R 中实现指针的方法。

让我们考虑下面的简单示例：

```
ep = rinterface.SexpExtPtr("hohoho")

```

python 字符串现在被封装到一个 r 外部指针中，并且可以被嵌入的 r 进程看到。

当考虑在 r 和 python 之间共享 C 级结构时，可以考虑更多涉及的示例（这里仍然是一个简单的示例）：

```
import ctypes
class Point2D(ctypes.Structure):
    _fields_ = [("x", ctypes.c_int),
                ("y", ctypes.c_int)]

pt = Point2D()

ep = rinterface.SexpExtPtr(pt)

```

但是，除非存在从 R 访问数据的方法，否则这仍然是一个相当学术的练习；当在 R 包中使用时，外部指针具有处理 C 级数据结构的伴随函数。

对于外部指针及其由 R 包定义的伴随函数和方法，RPY2 接口允许程序员直接从 python 创建此类外部指针，例如使用[`ctypes`](http://docs.python.org/library/ctypes.html#module-ctypes "(in Python v2.7)")。

然而，RPY2 接口允许的不止这些，因为程序员能够使 r has 访问的 python 函数本身就是一个函数。可以定义任意的 python 数据结构以及对其进行操作的函数或方法，将数据结构作为外部指针传递给 r，并将函数和方法公开给 r。

## 类图

## 其他变量

R_HOME

R HOME

`R_LEN_T_MAX`

largest usable integer for indexing R vectors

`TRUE`/`FALSE`

R’s TRUE and FALSE

### R 型

#### 矢量类型

`CPLXSXP`

Complex

`INTSXP`

Integer.

`LGLSXP`

Boolean (logical in the R terminology)

`RAWSXP`

Raw (bytes) value

`REALSXP`

Numerical value (float / double)

`STRSXP`

String

`VECSXP`

List

`LISTSXP`

Paired list

`LANGSXP`

Language object.

`EXPRSXP`

Unevaluated expression.

#### 函数类型

`CLOSXP`

Function with an enclosure. Represented by `rpy2.rinterface.SexpClosure`.

`BUILTINSXP`

Base function

`SPECIALSXP`

Some other kind of function

#### 其他类型

`ENVSXP`

Environment. Represented by `rpy2.rinterface.SexpEnvironment`.

`S4SXP`

Instance of class S4\. Represented by `rpy2.rinterface.SexpS4`.

#### 不应满足的类型

`PROMSXP`

Promise.