# `rlike`

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/rlike.html](https://rpy2.readthedocs.io/en/version_2.8.x/rlike.html)

## 概述

该包为纯 Python 上下文（即不运行嵌入式 R）提供 R 特性。

## 容器

该模块包含数据收集类型的数据结构。`OrdDict`和`TaggedList`是可以标记包含项/元素的结构。

模块可以导入如下：

```
>>> import rpy2.rlike.container as rlc

```

### 奥德迪克特

`OrdDict`提出了一个实现，在 python 中有时被称为有序的 dictionary，它有一个特殊性：键`None`意味着，尽管一个项有一个列组，并且可以从该列组中检索，但它没有“名称”。

为了简化它的使用，实现了[**pep 372**](https://www.python.org/dev/peps/pep-0372)中有序 dictionary 的 API。使用的一个例子是：

```
>>> x = (('a', 123), ('b', 456), ('c', 789))
>>> nl = rlc.OrdDict(x)

```

```
>>> nl['a']
123
>>> nl.index('a')
0

```

并非所有元素都必须命名，并且指定一个等于`None`的键值表示一个没有名称关联的值。

```
>>> nl[None] = 'no name'

```

类的 python docstring 是：

### 标签列表

`TaggedList`是一个 python[`list`](http://docs.python.org/library/functions.html#list "(in Python v2.7)")，其中每个项都有一个关联的`tag`。这类似于 R 中的`named`向量。

```
>>> tl = rlc.TaggedList([1,2,3])
>>> tl
[1, 2, 3]
>>> tl.tags()
(None, None, None)
>>> tl.settag(0, 'a')
>>> tl.tags()
('a', None, None)

```

```
>>> tl = rlc.TaggedList([1,2,3], tags=('a', 'b', 'c'))
>>> tl
[1, 2, 3]
>>> tl.tags()
('a', 'b', 'c')
>>> tl.settag(2, 'a')
>>> tl.tags()
('a', 'b', 'a')
>>> it = tl.iterontag('a')
>>> [x for x in it]
[1, 3]

```

```
>>> [(t, sum([i for i in tl.iterontag(t)])) for t in set(tl.itertags())]
[('a', 4), ('b', 2)]

```

The Python docstring for the class is:

## 使用序列的工具

在这里可以找到用于处理实现序列协议的对象的工具。

```
>>> import rpy2.rlike.functional as rlf
>>> rlf.tapply((1,2,3), ('a', 'b', 'a'), sum)
[('a', 4), ('b', 2)]

```

`TaggedList`对象可以与它们的标记一起使用（尽管使用它们的方法`iterontags()`可以实现更大的灵活性）：

```
>>> import rpy2.rlike.container as rlc
>>> tl = rlc.TaggedList([1, 2, 3], tags = ('a', 'b', 'a'))
>>> rlf.tapply(tl, tl.tags(), sum)
[('a', 4), ('b', 2)]

```

## 索引

许多 R 样式的索引都可以通过 Python 的列表理解来实现：

```
>>> l = ('a', 'b', 'c')
>>> l_i = (0, 2)
>>> [l[i] for i in l_i]
['a', 'c']

```

在`R`中，负索引意味着应该排除值。同样，可以使用列表理解（尽管这不是最有效的方法）：

```
>>> l = ('a', 'b', 'c')
>>> l_i = (-1, -2)
>>> [x for i, x in enumerate(l) if -i not in l_i]
['a']

```

`rpy2.rlike.indexing.``order`(_seq_, _cmp = default_cmp_, _reverse = False_)

给出顺序`seq`中项目的排序顺序，并对其进行排序。可选函数 cmp 应返回+1、-1 或 0。

&lt;colgroup&gt;&lt;col class="field-name"&gt; &lt;col class="field-body"&gt;&lt;/colgroup&gt;
| 参数： | 

*   **序列**–序列
*   **cmp**–函数
*   **反转**–布尔值

 |
| --- | --- |
| 返回类型： | 整数列表 |
| --- | --- |

```
>>> import rpy2.rlike.indexing as rli
>>> x = ('a', 'c', 'b')
>>> o = rli.order(x)
>>> o
[0, 2, 1]
>>> [x[i] for i in o]
['a', 'b', 'c']

```