# 回调

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/callbacks.html](https://rpy2.readthedocs.io/en/version_2.8.x/callbacks.html)

虽然 R 与控制台紧密相连，但是 R 核心开发团队在让前端开发人员根据需要定制 R 的交互行为方面取得了很大进展。

`rpy2`提供通过回调函数自定义 R 的交互行为。

## 控制台 I/O

在交互会话期间，R 和用户之间的大部分通信都是通过控制台进行的。控制台如何读取输入和写入输出，可以通过回调函数定义。

### 读取控制台

每当需要控制台输入时，都会调用“read console”函数。

输入数据的默认回调是`rinterface.consoleRead()`

一个合适的回调函数，比如它接受一个类为[`str`](http://docs.python.org/library/functions.html#str "(in Python v2.7)")的参数，即提示，并将用户输入返回为[`str`](http://docs.python.org/library/functions.html#str "(in Python v2.7)")。

函数对`rpy2.rinterface.set_readconsole()`和`rpy2.rinterface.get_readconsole()`分别用于设置和检索回调函数。

### 写入控制台

每当输出发送到 R 控制台时，都会调用“写入控制台”函数。

一个合适的回调函数是这样的：它接受一个类为[`str`](http://docs.python.org/library/functions.html#str "(in Python v2.7)")的参数，并且只具有副作用（不返回任何东西）。

函数对`rpy2.rinterface.set_writeconsole()`和`rpy2.rinterface.get_writeconsole()`分别用于设置和检索回调函数。

名为`rinterface.consolePrint()`的默认回调函数是对[`sys.stdout`](http://docs.python.org/library/sys.html#sys.stdout "(in Python v2.7)")的简单写入

一个例子应该很明显：

```
buf = []
def f(x):
    # function that append its argument to the list 'buf'
    buf.append(x)

# output from the R console will now be appended to the list 'buf'
rinterface.set_writeconsole(f)

date = rinterface.baseenv['date']
rprint = rinterface.baseenv['print']
rprint(date())

# the output is in our list (as defined in the function f above)
print(buf)

# restore default function
rinterface.set_writeconsole(rinterface.consolePrint)

```

### 冲洗控制台

The ‘write console’ function is called whenever output is sent to the R console.

合适的回调函数是这样的：它不接受任何参数，只具有副作用（不返回任何内容）。

函数对`rpy2.rinterface.set_flushconsole()`和`rpy2.rinterface.get_flushconsole()`分别用于设置和检索回调函数。

## 文件夹

### 显示文件

### 选择文件

在基本的 R 控制台上选择一个文件几乎没有铃铛和口哨声。

```
def choose_csv(prompt):
    print(prompt)
    return(filename)

```

## 其他

### 清理

当被要求终止时，通过其终端控制台 win32 或 quartz gui 前端，_r_ 将在开始时执行清理操作，无论用户是否要保存工作区。

在清理步骤中发生的事情可以通过一个回调函数指定，该函数将采用三个参数 _saveact_、_status_ 和 _runlast_、返回 1（保存工作区）、0（不保存工作区）和 none（取消退出/清理，引发一个`RRuntimeError`）。

```
import rpy2.rinterface

rpy2.rinterface.initr()

rquit = rpy2.rinterface.baseenv['q']

def cleanup(saveact, status, runlast):
    # cancel all attempts to quit R programmatically
    print("One can't escape...")
    return None

```

```
>>>  orig_cleanup = rpy2.rinterface.get_cleanup()
>>>  rpy2.rinterface.set_cleanup(cleanup)
>>> rquit()

```

恢复原始清理：

```
>>> rpy2.rinterface.set_cleanup(orig_cleanup)

```