# `rpy_classic`

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/rpy_classic.html](https://rpy2.readthedocs.io/en/version_2.8.x/rpy_classic.html)

该模块提供了一个与 rpy-1.x（_rpy_）中的 API_ 类似的 _。

警告

RPY-1.x 特性的实现不完整。这可能不是由于低级接口[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")的限制，而是由于作者的时间有限，以及为改进它所做的贡献有限。

为了匹配 _rpy_ 的示例和文档，我们将模块加载为：

```
>>> import rpy2.rpy_classic as rpy

```

## 转换

虽然[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")中建议的高级接口不需要明确的转换设置，但提供了 _rpy_ 中存在的转换系统，默认模式可以设置为`set_default_mode()`：

```
>>> rpy.set_default_mode(rpy.NO_CONVERSION)
>>> rpy.set_default_mode(rpy.BASIC_CONVERSION)

```

## R 实例

类`R`的`r`实例的行为如下：

```
>>> rpy.r.help

```

R 名称中的“点”转换为下划线：

```
>>> rpy.r.wilcox_test

```

```
>>> rpy.r.wilcox_test([1,2,3], [4,5,6])

```

```
>>> x = rpy.r.seq(1, 3, by=0.5)
>>> rpy.r.plot(x)

```

一个例子：

```
degrees = 4
grid = rpy.r.seq(0, 10, length=100)
values = [rpy.r.dchisq(x, degrees) for x in grid]
rpy.r.par(ann=0)
rpy.r.plot(grid, values, type='l')

rpy.r.library('splines')

type(rpy.r.seq)

```

## 功能

在 rpy-1.x 中，所有 r 对象都是可调用的：

```
>>> callable(rpy.r.seq)
True
>>> callable(rpy.r.pi)
True
>>>

```

如果一个对象不是 r 函数，那么在调用时 r 会抛出一个`RuntimeError`。

```
>>> rpy.r.pi()

```

函数的调用类似于常规的 python 函数：

```
>>> rpy.r.seq(1, 3)
>>> rpy.r.seq(1, 3, by=0.5)
>>> rpy.r['options'](show_coef_Pvalues=0)
>>>

```

```
>>> m = rpy.r.matrix(r.rnorm(100), 20, 5)
>>> pca = rpy.r.princomp(m)
>>> rpy.r.plot(pca, main = "PCA")
>>>

```

## 部分使用`rpy_classic`

rpy_classic 的使用不需要排除 rpy2 中建议的其他接口。

为任何一个接口设计的链接代码都相当容易，而且，在其他可能的用例中，将遗留的 RPY 代码包含到新编写的 RPY2 代码中应该是一个简单的方法。

`rpy_classic`与`rpy2`其余部分之间的链接是属性`RObj.sexp`，它给出底层[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")定义中底层 r 对象的表示。然后可以在带有[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")和[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")的函数调用中使用此表示。使用[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")可以考虑使用`rpy2.robjects.default_ri2py()`进行转换。

注意

显然，该属性`sexp`不是 RPY 中原始`Robj`的一部分。

An example:

```
import rpy2.robjects as ro
import rpy2.rpy_classic as rpy
rpy.set_default_mode(rpy.NO_CONVERSION)

def legacy_paste(v):
    # legacy rpy code
    res = rpy.r.paste(v, collapse = '-')
    return res

rletters = ro.r['letters']

# the legaxy code is called using an rpy2.robjects object
alphabet_rpy = legacy_paste(rletters)

# convert the resulting rpy2.rpy_classic object to
# an rpy2.robjects object
alphabet = ro.default_ri2py(alphabet_rpy.sexp)

```