# 自定义图形设备

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/graphicaldevices.html](https://rpy2.readthedocs.io/en/version_2.8.x/graphicaldevices.html)

警告

这仍然是非常实验性的，使用它可能会导致 Python 解释器崩溃。

C-API 到 R 允许扩展编写器实现自定义图形设备（使用 C）。例如，此功能用于实现 SVG 或 cairo 的驱动程序（cairo 支持稍后将其提供给 r codebase）。

RPY2 向 Python 程序员公开了客户图形化工具的创建，而不需要 C。

为了演示如何实现图形化，我们考虑以下示例：一个计算使用图形原语次数的设备。这是非常有限的实际应用，但足以解释这些原则。

这种装置的实施方式如下：

```
import rpy2.rinterface._rpy_device as rdevice
from collections import Counter

class BeancounterDevice(rdevice.GraphicalDevice):
    """ Graphical devive for R that counts the
    number of times primitives are called."""

    def __init__(self):
        super(BeancounterDevice, self).__init__()
        self._ct = Counter()

    def circle(self, x, y, radius):
        self._ct['circle'] += 1

    def clip(self, x0, x1, y0, y1):
        self._ct['clip'] += 1

    def line(self, x1, y1, x2, y2):
        self._ct['lines'] += 1

    def mode(self, mode):
        self._ct['mode'] += 1

    def rect(self, x0, x1, y0, y1):
        self._ct['rectangle'] += 1

    def strwidth(self, text):
        self._ct['strwidth'] += 1
        return float(0)

    def text(x, y, string, rot, hadj):
        self._ct['text'] += 1

```

类`BeancounterDevice`现在可以用作真正的 R 绘图设备。

```
from rpy2.robjects.packages import importr

dev = BeancounterDevice()

graphics = importr("graphics")
# plot into our counting device
graphics.plot(0, 0)

# Print the counts
print(dev._ct)

```

要为 R 实现新的自定义图形设备，只需扩展类`rpy2.rinterface._rpy_device.GraphicalDevice`。如果新设备不实现 R 所使用的功能，则会打印错误消息。

类及其方法的 python 文档字符串为：