# RPY2 简介

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/introduction.html](https://rpy2.readthedocs.io/en/version_2.8.x/introduction.html)

本介绍面向新用户，或从未查阅过文档但在通过 API 成功猜测其第一步后遇到拦截器的用户。

## 开始

这里假设 RPY2 包已正确安装。如果使用一个可用的 Docker 容器，或者按照说明操作（请参见[安装](overview.html#install-installation)），则会出现这种情况。

`rpy2`与绑定到 C 库的任何其他 python 包一样。它的顶层可以导入，并获得版本。

```
import rpy2
print(rpy2.__version__)

```

注意

当用 RPY2 报告问题时，RPY2 版本相当重要，或者在试图评估 RPY2 是否与预期版本匹配时，用您自己的代码报告问题时，RPY2 版本相当重要。

生成（编译）rpy2 所依据的 r 版本也可用：

```
from rpy2.rinterface import R_VERSION_BUILD
print(R_VERSION_BUILD)

```

`rpy2`提供 2 级与 R 的接口：—低层（[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")）—高层（[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")）

高级接口正试图尽可能自然地为 Python 用户（有时称为“pythonic”）使用 r，而本文的介绍仅涵盖了该接口。

导入顶级子包也正在初始化并启动嵌入在当前 python 进程中的 r：

```
import rpy2.robjects as robjects

```

## R 包

由于包的广度和深度，R 可以说是最好的数据分析工具箱之一。

### 正在导入包

导入 r 包通常是运行 r 代码时的第一步，而`rpy2`提供的函数`rpy2.robjects.packages.importr()`使该步骤与导入 python 包非常相似。

```
from rpy2.robjects.packages import importr
# import R's "base" package
base = importr('base')

# import R's "utils" package
utils = importr('utils')

```

实际上，这一步是在嵌入式 R 中导入 R 包，并将该包中的所有 R 对象作为 Python 对象公开。

Note

不过还有一个问题。r 对象名可以包含一个`”.”`（dot），而在 python 中，dot 表示“命名空间中的属性”。因此，`importr`试图将`”.”`翻译成`“_”`。在大多数情况下，这些细节是不必要的，但是当它们做这些时，应该参考[R 包](robjects_rpackages.html#robjects-packages)的文档。

### 安装程序包

知道如何安装 R 软件包是必须具备的一项重要技能，但如果设计出一个 R 软件包来满足任务或项目的所有合理需求，则不一定总是强制性的。

Note

包安装在介绍的早期就已经介绍过了，但是如果遇到诸如缺少 Internet 连接、不合作的代理（或代理维护者）或安装包的写入权限不足等困难，则可以跳过本小节。

下载和安装 R 包通常通过从包存储库中获取 R 包并在本地安装来执行。这样做的功能由 R 库提供，在 Python 中，我们可以使用 RPY2 简单地使用它们。在`rpy2.robjects.packages`中提供了 R 功能的接口（定义了上面介绍的函数`importr()`）。

准备好从 r 已知的第一个镜像安装软件包是用以下方法完成的：

```
# import rpy2's package module
import rpy2.robjects.packages as rpackages

# import R's utility package
utils = rpackages.importr('utils')

# select a mirror for R packages
utils.chooseCRANmirror(ind=1) # select the first mirror in the list

```

我们现在准备使用 r 自己的函数`install.package`安装包：

```
# R package names
packnames = ('ggplot2', 'hexbin')

# R vector of strings
from rpy2.robjects.vectors import StrVector

# Selectively install what needs to be install.
# We are fancy, just because we can.
names_to_install = [x for packnames if not rpackages.isinstalled(x)]
if len(names_to_install) > 0:
    utils.install_packages(StrVector(names_to_install))

```

如果您依赖默认情况下不与 R 一起分发的 CRAN 包，那么上面的代码可以是您分发的 Python 代码的一部分。

有关在`rpy2`中处理 R 包的更多文档，请参见[R 包](robjects_rpackages.html#robjects-packages)一节。

## `r`实例

我们之前提到过`rpy2`正在运行一个嵌入式 R。这可能有点抽象，所以有一个对象`rpy2.robjects.r`使其成为有形的。

这个对象可以用作 python 和 r 之间的基本通信通道，类似于一个与子进程交互的方式，更高效，更好地与 python 集成，更易于使用。

### 获取 r 对象

`rpy2.robjects.r`的`__getitem__()`方法获取与给定符号关联的 r 对象，就像在 r 控制台中键入该符号名称一样（有关详细信息，请参见下面的注释）。

R 中的示例：

```
> pi
[1] 3.141593

```

使用`rpy2`：

```
>>> pi = robjects.r['pi']
>>> pi[0]
3.14159265358979

```

Note

在 hood 下，变量`pi`默认从 r_ 基 _ 包中获取，除非在 r 的`.globalEnv`中创建了另一个名为`pi`的变量。

无论何时，只要想明确寻找符号的位置（大多数时候都是这样），就可以在 python 名称空间对象中包装 r 包（请参见[r packages](robjects_rpackages.html#robjects-packages)）。

有关环境的详细信息，请参阅[环境](robjects_environments.html#robjects-environments)一节。

另外，请注意 _pi_ 不是标量，而是长度为 1 的向量。

### 正在评估 R 代码

对象`r`也是可调用的，调用中传递的字符串计算为`R`代码。

最简单的字符串是 r 对象的名称，这提供了前面描述的方法`__getitem__`的替代方法。

Example in R:

```
> pi
[1] 3.141593

```

With `rpy2`:

```
>>> pi = robjects.r('pi')
>>> pi[0]
3.14159265358979

```

警告

结果是一个 r 向量。下面的[r 向量](#introduction-vectors)部分将解释以下行为：

```
>>> piplus2 = robjects.r('pi') + 2
>>> piplus2.r_repr()
c(3.14159265358979, 2)
>>> pi0plus2 = robjects.r('pi')[0] + 2
>>> print(pi0plus2)
5.1415926535897931

```

更复杂的字符串是任意复杂度的 r 表达式，甚至是表达式序列（r 代码片段）。他们的评估是在 R 用户所知道的`Global Environment`中执行的，也就是在 R 控制台中开始的地方。每当`R`代码创建变量时，默认情况下这些变量都位于该`Global Environment`中。

例如，下面的字符串返回值 18.85。

```
robjects.r('''
        # create a function `f`
        f <- function(r, verbose=FALSE) {
            if (verbose) {
                cat("I am calling f().\n")
            }
            2 * pi * r
        }
        # call the function `f` with argument value 3
        f(3)
        ''')

```

该字符串是一段 r 代码（包含注释），它首先创建一个 r 函数，然后将其绑定到符号`f`（在 r 中），最后调用该函数`f`。调用的结果（r 函数`f`返回的结果）返回给 python。

由于该函数`f`现在出现在 r`Global Environment`中，因此可以使用上面概述的`__getitem__`机制访问它：

```
>>> r_f = robjects.globalenv['f']
>>> print(r_f.r_repr())
function (r, verbose = FALSE)
{
    if (verbose) {
        cat("I am calling f().\n")
    }
    2 * pi * r
}

```

Note

如前所示，获取函数的另一种方法是从`R`单例

```
>>> r_f = robjects.r['f']

```

函数 r_f 是可调用的，可以像普通的 python 函数一样使用。

```
>>> res = r_f(3)

```

有关调用函数的详细信息，请跳到调用 r 函数部分。

### 将 r 对象插入 r 代码字符串

根据本节标题的第一印象，这里介绍了`rpy2`的简单和方便的功能。

一个 R 对象有一个字符串表示，可以直接使用到要评估的 R 代码中。

简单示例：

```
>>> letters = robjects.r['letters']
>>> rcode = 'paste(%s, collapse="-")' %(letters.r_repr())
>>> res = robjects.r(rcode)
>>> print(res)
"a-b-c-d-e-f-g-h-i-j-k-l-m-n-o-p-q-r-s-t-u-v-w-x-y-z"

```

## R 向量

在`R`中，数据主要由向量表示，即使看起来像标量。

当仔细观察之前使用的 r 对象`pi`时，我们可以观察到这实际上是一个长度为 1 的向量。

```
>>> len(robjects.r['pi'])
1

```

因此，python 方法`add()`将导致串联（r 中的函数`c()`），因为常规 python 列表就是这样。

访问该向量中的一个值必须明确说明：

```
>>> robjects.r['pi'][0]
3.1415926535897931

```

向量可以实现很多功能，让它们的行为更像 python 列表或 r 向量。矢量行为的全面描述见`robjects.vector`。

### 创建 rpy2 向量

创建 R 向量可以简单地实现：

```
>>> res = robjects.StrVector(['abc', 'def'])
>>> print(res.r_repr())
c("abc", "def")
>>> res = robjects.IntVector([1, 2, 3])
>>> print(res.r_repr())
1:3
>>> res = robjects.FloatVector([1.1, 2.2, 3.3])
>>> print(res.r_repr())
c(1.1, 2.2, 3.3)

```

r 矩阵和数组只是具有`dim`属性的向量。

创建此类对象的最简单方法是通过 r 函数：

```
>>> v = robjects.FloatVector([1.1, 2.2, 3.3, 4.4, 5.5, 6.6])
>>> m = robjects.r['matrix'](v, nrow = 2)
>>> print(m)
     [,1] [,2] [,3]
[1,]  1.1  3.3  5.5
[2,]  2.2  4.4  6.6

```

## 调用 r 函数

调用 r 函数与调用 python 函数非常相似：

```
>>> rsum = robjects.r['sum']
>>> rsum(robjects.IntVector([1,2,3]))[0]
6L

```

关键词也起作用：

```
>>> rsort = robjects.r['sort']
>>> res = rsort(robjects.IntVector([1,2,3]), decreasing=True)
>>> print(res.r_repr())
c(3L, 2L, 1L)

```

Note

默认情况下，调用 r 函数返回 r 对象。

有关函数的更多信息，请参见[函数](robjects_functions.html#robjects-functions)部分。

## 获取帮助

r 有一个内置的帮助系统，就像 pydoc 字符串在交互会话期间在 python 中经常使用一样，r 编程人员经常使用它。此帮助系统可从 R 函数访问，因此可从 RPY2 访问。

### 有关给定包或当前加载的包中的主题的帮助

```
>>> from rpy2.robjects.packages import importr
>>> utils = importr("utils")
>>> help_doc = utils.help("help")
>>> help_doc[0]
'/where/R/is/installed/library/utils/help/help'

```

将返回的对象转换为字符串将生成有关主题的完整帮助文本：

```
>>> str(help_doc)
[...long output...]

```

Warning

这样生成的帮助消息不是返回到控制台的字符串，而是由 R 直接打印到标准输出。对`str()`的调用只返回一个空字符串，其原因在介绍性文档中有所涉及。这种行为植根于**r**本身，而在`rpy2`中，r 对象的字符串表示形式是由**r**控制台给出的字符串表示形式，在这种情况下，该控制台采用单一路由。

对于 R 帮助系统的 python 友好帮助，请考虑模块[`rpy2.robjects.help`](rhelp.html#module-rpy2.robjects.help "rpy2.robjects.help: High-level interface with R (Unix, Windows)")。

### 在可用包中查找主题

```
>>> help_where = utils.help_search("help")

```

与以前使用`help`一样，可以将结果打印/转换为字符串，得到与从 R 会话获得的结果类似的结果。

Note

返回的数据结构可以用于访问详细返回的信息。

```
>>> tuple(help_where)
(<StrVector - Python:0x1f9a968 / R:0x247f908>,
 <StrVector - Python:0x1f9a990 / R:0x25079d0>,
 <StrVector - Python:0x1f9a9b8 / R:0x247f928>,
 <Matrix - Python:0x1f9a850 / R:0x1ec0390>)
>>> tuple(help_where[3].colnames)
('topic', 'title', 'Package', 'LibPath')

```

但是，这超出了介绍的范围，您应该先掌握模块`robjects.vector`的内容，然后再掌握其他内容。

## 示例

本节演示 RPY2 的一些特性。

### 图形和绘图

```
import rpy2.robjects as robjects

r = robjects.r

x = robjects.IntVector(range(10))
y = r.rnorm(10)

r.X11()

r.layout(r.matrix(robjects.IntVector([1,2,3,2]), nrow=2, ncol=2))
r.plot(r.runif(10), y, xlab="runif", ylab="foo/bar", col="red")

```

在函数调用中动态设置参数的数量可以用 Python 中的常规方法来完成。

在`R`中有几种绘制数据的方法，其中一些方法在本文档中介绍：

此处重复常规设置：

```
from rpy2 import robjects
from rpy2.robjects import Formula, Environment
from rpy2.robjects.vectors import IntVector, FloatVector
from rpy2.robjects.lib import grid
from rpy2.robjects.packages import importr, data
from rpy2.rinterface import RRuntimeError
import warnings

# The R 'print' function
rprint = robjects.globalenv.get("print")
stats = importr('stats')
grdevices = importr('grDevices')
base = importr('base')
datasets = importr('datasets')

grid.activate()

```

特定于 ggplot2 的设置是：

```
import math, datetime
import rpy2.robjects.lib.ggplot2 as ggplot2
import rpy2.robjects as ro
from rpy2.robjects.packages import importr
base = importr('base')

mtcars = data(datasets).fetch('mtcars')['mtcars']

```

```
pp = ggplot2.ggplot(mtcars) + \
     ggplot2.aes_string(x='wt', y='mpg', col='factor(cyl)') + \
     ggplot2.geom_point() + \
     ggplot2.geom_smooth(ggplot2.aes_string(group = 'cyl'),
                         method = 'lm')
pp.plot()

```

[![_images/graphics_ggplot2_smoothbycylwithcolours.png](img/a5ac0c7487f6ebb043c5418553166c9f.jpg)](_images/graphics_ggplot2_smoothbycylwithcolours.png)

有关 R 中的绘图和图形的更多信息，以及更高级的绘图，请参见[图形](graphics.html#graphics)部分。

Warning

默认情况下，嵌入的 R 打开一个交互式绘图设备，即绘图所在的窗口。必须明确要求在该设备上处理交互式事件，例如调整大小或关闭窗口（请参见[处理交互式事件](rinterface.html#rinterface-interactive-processevents)一节）。

### 线性模型

R 代码为：

```
ctl <- c(4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14)
trt <- c(4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69)
group <- gl(2, 10, 20, labels = c("Ctl","Trt"))
weight <- c(ctl, trt)

anova(lm.D9 <- lm(weight ~ group))

summary(lm.D90 <- lm(weight ~ group - 1))# omitting intercept

```

使用[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")实现相同目标的一种方法是

```
from rpy2.robjects import FloatVector
from rpy2.robjects.packages import importr
stats = importr('stats')
base = importr('base')

ctl = FloatVector([4.17,5.58,5.18,6.11,4.50,4.61,5.17,4.53,5.33,5.14])
trt = FloatVector([4.81,4.17,4.41,3.59,5.87,3.83,6.03,4.89,4.32,4.69])
group = base.gl(2, 10, 20, labels = ["Ctl","Trt"])
weight = ctl + trt

robjects.globalenv["weight"] = weight
robjects.globalenv["group"] = group
lm_D9 = stats.lm("weight ~ group")
print(stats.anova(lm_D9))

# omitting the intercept
lm_D90 = stats.lm("weight ~ group - 1")
print(base.summary(lm_D90))

```

这种执行线性拟合的方法与上面提到的 r 中的方法精确匹配，但还有其他方法（参见[公式](robjects_formulae.html#robjects-formula)一节，将变量直接存储在公式的查找环境中）。

问：现在如何从生成的对象中提取数据？

A：嗯，这完全取决于物体。R 非常适合交互式会话，用户经常检查函数返回的内容，以了解如何提取信息。

当从上面的代码中获取结果时，可以这样做：

```
>>> print(lm_D9.rclass)
[1] "lm"

```

这里的结果对象是一个列表结构，因为检查数据结构或读取`lm`的 R 手册页都会告诉我们。检查它的元素名是很简单的：

```
>>> print(lm_D9.names)
 [1] "coefficients"  "residuals"     "effects"       "rank"
 [5] "fitted.values" "assign"        "qr"            "df.residual"
 [9] "contrasts"     "xlevels"       "call"          "terms"
[13] "model"

```

提取一个特定元素也是如此：

```
>>> print(lm_D9.rx2('coefficients'))
(Intercept)    groupTrt
      5.032      -0.371

```

或

```
>>> print(lm_D9.rx('coefficients'))
$coefficients
(Intercept)    groupTrt
      5.032      -0.371

```

有关从向量中提取元素的详细信息，请参见[提取项](vector.html#robjects-extracting)。

### 主成分分析

R 代码是

```
m <- matrix(rnorm(100), ncol=5)
pca <- princomp(m)
plot(pca, main="Eigen values")
biplot(pca, main="biplot")

```

[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")代码可以尽可能接近 R 代码：

```
import rpy2.robjects as robjects

r = robjects.r

m = r.matrix(r.rnorm(100), ncol=5)
pca = r.princomp(m)
r.plot(pca, main="Eigen values")
r.biplot(pca, main="biplot")

```

但是，同样的例子可以做得更整洁一点（关于使用的 r 函数的具体情况）

```
from rpy2.robjects.packages import importr

base     = importr('base')
stats    = importr('stats')
graphics = importr('graphics')

m = base.matrix(stats.rnorm(100), ncol = 5)
pca = stats.princomp(m)
graphics.plot(pca, main = "Eigen values")
stats.biplot(pca, main = "biplot")

```

### 创建 R 向量或矩阵，并使用 python 代码填充其单元格

```
from rpy2.robjects import NA_Real
from rpy2.rlike.container import TaggedList
from rpy2.robjects.packages import importr

base = importr('base')

# create a numerical matrix of size 100x10 filled with NAs
m = base.matrix(NA_Real, nrow=100, ncol=10)

# fill the matrix
for row_i in xrange(1, 100+1):
    for col_i in xrange(1, 10+1):
        m.rx[TaggedList((row_i, ), (col_i, ))] = row_i + col_i * 100

```

```
None

```

### 再举一个例子

```
"""
short demo.

"""

from rpy2.robjects.packages import importr
graphics = importr('graphics')
grdevices = importr('grDevices')
base = importr('base')
stats = importr('stats')

import array

x = array.array('i', range(10))
y = stats.rnorm(10)

grdevices.X11()

graphics.par(mfrow = array.array('i', [2,2]))
graphics.plot(x, y, ylab = "foo/bar", col = "red")

kwargs = {'ylab':"foo/bar", 'type':"b", 'col':"blue", 'log':"x"}
graphics.plot(x, y, **kwargs)

m = base.matrix(stats.rnorm(100), ncol=5)
pca = stats.princomp(m)
graphics.plot(pca, main="Eigen values")
stats.biplot(pca, main="biplot")

```