# 将代码移植到 RPY2

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/porting-to-rpy2.html](https://rpy2.readthedocs.io/en/version_2.8.x/porting-to-rpy2.html)

## 从 R

## 来自 RPY

### 兼容层

存在一个兼容层，尽管它目前没有完全实现 RPY 接口。

### 忠实的例子

多年来，蒂姆·丘奇（TimChurch）的 _ 老忠实 _ 的例子似乎已经成为许多`rpy`用户的标志性地位。这个例子是罗塞塔石碑的明显文本，我们将其翻译成 rpy2-2.1.0 的[`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)")。此示例基于 John A.Schroeder 对 RPY2-2.0.8 的翻译（也适用于版本 2.1，但由于明显的兼容性原因，无法使用新功能）。

#### 设置

RPY2 可以隐藏更多的 R 层，提供只需要 Python 知识的接口。

```
from rpy2.robjects.vectors import DataFrame
from rpy2.robjects.packages import importr, data

r_base = importr('base')

```

该示例仅显式使用`rpy2.robjects.vectors.DataFrame`和定义的 r 包。函数`rpy2.robjects.packages.importr()`允许将包中的 r 函数封装成一种对 python 友好的形式。

#### 导入数据

```
faithful_data = DataFrame.from_csvfile('faithful.dat', sep = " ")

```

如果附近没有数据文件，则可以从 R 自己的数据集集合加载此数据集：

```
datasets = importr('datasets')
faithful_data = data(datasets).fetch('faithful')['faithful']

```

#### 总结

```
edsummary = r_base.summary(faithful_data.rx2("eruptions"))
for k, v in edsummary.items():
   print("%s: %.3f\n" %(k, v))

```

#### 茎叶图

```
graphics = importr('graphics')

print("Stem-and-leaf plot of Old Faithful eruption duration data")
graphics.stem(faithful_data.rx2("eruptions"))

```

#### 柱状图

```
grdevices = importr('grDevices')
stats = importr('stats')
grdevices.png('faithful_histogram.png', width = 733, height = 550)
ed = faithful_data.rx2("eruptions")
graphics.hist(ed, r_base.seq(1.6, 5.2, 0.2),
              prob = True, col = "lightblue",
              main = "Old Faithful eruptions", xlab = "Eruption duration (seconds)")
graphics.lines(stats.density(ed,bw=0.1), col = "orange")
graphics.rug(ed)
grdevices.dev_off()

```

或者，可以使用 ggplot2 包来绘制：

```
from rpy2.robjects.lib import ggplot2

p = ggplot2.ggplot(faithful_data) + \
    ggplot2.aes_string(x = "eruptions") + \
    ggplot2.geom_histogram(fill = "lightblue") + \
    ggplot2.geom_density(ggplot2.aes_string(y = '..count..'), colour = "orange") + \
    ggplot2.geom_rug() + \
    ggplot2.scale_x_continuous("Eruption duration (seconds)") + \
    ggplot2.opts(title = "Old Faithful eruptions")

p.plot()

```

```
from rpy2.robjects.vectors import FloatVector

long_ed = FloatVector([x for x in ed if x > 3])
grdevices.png('faithful_ecdf.png', width = 733, height = 550)

stats = importr('stats')

params = {'do.points' : False,
          'verticals' : 1,
          'main' : "Empirical cumulative distribution function of " + \
                    "Old Faithful eruptions longer than 3 seconds"}
graphics.plot(stats.ecdf(long_ed), **params)
x = r_base.seq(3, 5.4, 0.01)
graphics.lines(x, stats.pnorm(x, mean = r_base.mean(long_ed),
                              sd = r_base.sqrt(stats.var(long_ed))),
               lty = 3, lwd = 2, col = "salmon")
grdevices.dev_off()

```

```
grdevices.png('faithful_qq.png', width = 733, height = 550)
graphics.par(pty="s")
stats.qqnorm(long_ed,col="blue")
stats.qqline(long_ed,col="red") # strangely in stats, not in graphics
grdevices.dev_off()

```

## 自 RPY2-2.0.x

本节涉及`rpy2.objects`层中的更改。如果对较低级别[`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)")的变更感兴趣，应参考附录中的变更列表。

### 骆驼壳

camelcase 的命名从变量和方法中消失了，因为在标准库中的这些 obejct 中似乎大多没有这种命名（尽管在[**pep 8**](https://www.python.org/dev/peps/pep-0008)中没有具体的命名）。

实际上，这意味着发生了以下名称更改：

<colgroup><col width="63%"> <col width="37%"></colgroup> 
| 旧名称 | 新名称 |
| --- | --- |
| [`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)") |
| `globalEnv` | `globalenv` |
| `baseNameSpaceEnv` | `baseenv` |
| [`rpy2.rinterface`](rinterface.html#module-rpy2.rinterface "rpy2.rinterface: Low-level interface with R (Unix, Windows)") |
| `globalEnv` | `globalenv` |
| `baseEnv` | `baseenv` |

## R 前缀类名

以字母`R`为前缀的类名已从该前缀中清除。例如，`RVector`变为`Vector`、`RDataFrame`变为`DataFrame`等…

<colgroup><col width="52%"> <col width="48%"></colgroup> 
| old name | new name |
| --- | --- |
| [`rpy2.robjects`](robjects.html#module-rpy2.robjects "rpy2.robjects: High-level interface with R (Unix, Windows)") |
| `RVector` | `Vector` |
| `RArray` | `Array` |
| `RMatrix` | `Matrix` |
| `RDataFrame` | `DataFrame` |
| `REnvironment` | `Environment` |
| `RFunction` | `Function` |
| `RFormula` | `Formula` |

### R 包的命名空间

函数`rpy2.robjects.packages.importr()`应用于将 r 包的名称空间作为一个 Python 友好对象导入。

```
>>> from rpy2.robjects.packages import importr
>>> base = importr("base")
>>> base.letters[0]
'a'

```

只要有可能，这个步骤就会安全地将 r 变量名中的“.”转换为 python 变量名的“.”。

[r packages](robjects_rpackages.html#robjects-packages)部分中的文档提供了更多详细信息。

## 函数调用中的参数名

默认情况下，暴露的 R 函数将尝试安全地转换其命名参数（“”将变为“”）。应检查[部分函数](robjects_functions.html#robjects-functions)的详细信息。

## 缺少值

r 有一个内置的概念，即 _ 缺少值 _，以及缺少值的类型。现在可以更好地集成到 RPY2 中（请参阅有关[缺少值](rinterface.html#missing-values)的更多信息）

## 绘图

将名称空间用于 R 包（参见上文），以及自定义表示一些特定 R 库，这使得图形（甚至）的生成变得更容易（参见[图形](graphics.html#graphics)一节）。