# `r`：R 的实例

> 原文：[https://rpy2.readthedocs.io/en/version_2.8.x/robjects_rinstance.html](https://rpy2.readthedocs.io/en/version_2.8.x/robjects_rinstance.html)

此类当前是一个单例类，在加载模块时会显示它的一个表示：

```
>>> robjects.r
>>> print(robjects.r)

```

实例可以看作是嵌入式 R 进程的入口点。

作为单一实例意味着每次调用`R`的构造函数时都会返回同一个实例；这是由于嵌入的 R 是有状态的。

可以从等效的 R 环境中访问的元素可以作为实例的属性进行访问。熟悉 python 的[`ctypes`](http://docs.python.org/library/ctypes.html#module-ctypes "(in Python v2.7)")模块的读者会注意到它的相似性。

R 矢量：

```
>>> pi = robjects.r.pi
>>> letters = robjects.r.letters

```

R 功能：

```
>>> plot = robjects.r.plot
>>> dir = robjects.r.dir

```

这种方法有以下限制：

*   对象的实际 python 属性掩盖了 r 元素

*   ‘.’ (dot) is syntactically valid in names for R objects, but not for

    python 对象。

如果这项功能对您最重要，那么使用[`rpy2.rpy_classic`](rpy_classic.html#module-rpy2.rpy_classic "rpy2.rpy_classic: Emulate the orignal rpy")可以部分消除最后的限制。

```
>>> robjects.r.as_null
# AttributeError raised
>>> import rpy2.rpy_classic as rpy
>>> rpy.set_default_mode(rpy.NO_CONVERSION)
>>> rpy.r.as_null
# R function as.null() returned

```

注意

部分使用 rpy_classic 一节概述了如何集成[`rpy2.rpy_classic`](rpy_classic.html#module-rpy2.rpy_classic "rpy2.rpy_classic: Emulate the orignal rpy")代码。

在场景后面，获取`r`属性的步骤相当简单：

> 1.  检查属性是否在`r`
> 2.  的 python 定义中定义，检查属性是否可以在 r 中访问，从`globalenv`
> 
> 开始

当安全最重要时，我们建议使用`__getitem__()`获取给定的 R 对象。

```
>>> as_null = robjects.r['as.null']

```

将对象存储在 python 变量中可以防止垃圾收集，即使是从 r 用户可见的对象中删除的。

```
>>> robjects.globalenv['foo'] = 1.2
>>> foo = robjects.r['foo']
>>> foo[0]
1.2

```

这里我们`remove`是来自 R 全球环境的符号`foo`。

```
>>> robjects.r['rm']('foo')
>>> robjects.r['foo']
LookupError: 'foo' not found

```

在从 python 中删除`foo`之前，对象本身仍然可用，并受 R 垃圾收集的保护。

```
>>> foo[0]
1.2

```

## 将字符串计算为 R 代码

正如 rpy-1.x 的情况一样，可以通过调用`r`实例来执行字符串中包含的 r 代码的动态评估：

```
>>> print(robjects.r('1+2'))
[1] 3
>>> sqr = robjects.r('function(x) x^2')

```

```
>>> print(sqr)
function (x)
x^2
>>> print(sqr(2))
[1] 4

```

精明的读者会很快意识到，用 python 变量命名的 r 对象可以通过它们的**r**表示插入到代码中：

```
>>> x = robjects.r.rnorm(100)
>>> robjects.r('hist(%s, xlab="x", main="hist(x)")' %x.r_repr())

```

警告

对大型对象执行此操作可能不是对计算能力的最佳利用。